/**@name LIBRARY: cietConfig_EH.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 09 08 2005

Modification History: 

	. 22/06/07		Pablo: 	Changed format for threshold, thermoCoupleOffset and heaterResistance to 3d.
	. 21/02/07		Pablo: 	Changed curveId from description to Dp.
	. 19/02/07		Pablo		Max and Min Range for Engineering values, taken from import file.
	. 16/02/07		Pablo		Added Range Trend configuration.
	.	18/12/06		Pablo		Changed format resistanceCalc (#.####)
	. 07/12/06	  Quique  Change formats and units in the faceplate
	. 29/09/06		Quique	Correct Bug setUnit: replace thCoupleTempeature by thCoupleTemparatureCalc
	.	25/09/06		Quique  Include UNIT in the thcouple values 
	. 21/08/06		Quique	Include UNIT read from specs file

version 1.0

External Function : 
	. cietConfig_EH_setConfig :  set  configuration of given DI datapoint
	. cietConfig_EH_checkConfig : check data before configuration
	
Internal Functions :
	. CRYOGTW_EH_checkConfig : 
	. CRYOGTW_EH_setConfig :
	
Purpose: 
This library contains the function used to set ciet EH configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//Constants

//@{
//------------------------------------------------------------------------------------------------------------------------

// cietConfig_EH_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_EH_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
//	DebugN("unConfigGenericFunctions_checkParameters");
		unConfigGenericFunctions_checkParameters(configLine, "EH", exceptionInfo);
	// 2. call the device check function
//		DebugN("unConfigGenericFunctions_checkDeviceConfig");
		unConfigGenericFunctions_checkDeviceConfig(configLine, "EH", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigEH_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfig_EH_setConfig
/**
Purpose: set configuration of given EH datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_EH_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "EH", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "EH", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "EH", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_EH_checkConfig
/**
Purpose: check data before configuration for CRYOGTW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CRYOGTW_EH_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;

	if (dynlen(configLine) == (ciet_CONFIG_EH_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (ciet_CONFIG_EH_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
//				DebugN("unConfigGenericFunctions_checkArchive");
		unConfigGenericFunctions_checkArchive(configLine[ciet_CONFIG_EH_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[ciet_CONFIG_EH_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[ciet_CONFIG_EH_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
//	DebugN("unConfigGenericFunctions_checkDeadBand");
	/*	unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_heatVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_heatRefVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_shuntVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_shuntRefVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thRefVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);		
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_HYSTERESIS], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_CYCLEPERIOD], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);	
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_suppliedVoltageCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_suppliedCurrentCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);		
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thCoupleVoltageCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thCoupleTemperatureCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);																
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_powerCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE], exceptionInfo);					*/																																																																																																										
//		DebugN("unConfigGenericFunctions_checkArchiveParameters");
		unConfigGenericFunctions_checkArchiveParameters(0, configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EH_ARCHIVEACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CRYOGTW_EH_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_EH_setConfig
/**
Purpose:  set configuration of given EH datapoint for CRYOGTW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

CRYOGTW_EH_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sCMWDeviceName;
	dyn_string dsDpe, dsDescription;
	int i;
	bool bArchive;
	int iArchiveTimeFilter, iArchiveType, iSmoothProcedure;
	dyn_float dfMax = makeDynFloat(ciet_CONFIG_EH_MAXVALUE_heatVolMeas, ciet_CONFIG_EH_MAXVALUE_heatRefVolMeas,
																	ciet_CONFIG_EH_MAXVALUE_shuntVolMeas, ciet_CONFIG_EH_MAXVALUE_shuntRefVolMeas,
																	ciet_CONFIG_EH_MAXVALUE_thVolMeas, ciet_CONFIG_EH_MAXVALUE_thRefVolMeas,
																	ciet_CONFIG_EH_MAXVALUE_HYSTERESIS, ciet_CONFIG_EH_MAXVALUE_CYCLEPERIOD,
																	ciet_CONFIG_EH_MAXVALUE_suppliedVoltageCalc, ciet_CONFIG_EH_MAXVALUE_suppliedCurrentCalc,
																 	ciet_CONFIG_EH_MAXVALUE_thCoupleVoltageCalc, ciet_CONFIG_EH_MAXVALUE_thCoupleTemperatureCalc, 
																 	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX]);
	dyn_float dfMin = makeDynFloat(ciet_CONFIG_EH_MINVALUE_heatVolMeas, ciet_CONFIG_EH_MINVALUE_heatRefVolMeas,
																	ciet_CONFIG_EH_MINVALUE_shuntVolMeas, ciet_CONFIG_EH_MINVALUE_shuntRefVolMeas,
																	ciet_CONFIG_EH_MINVALUE_thVolMeas, ciet_CONFIG_EH_MINVALUE_thRefVolMeas,
																	ciet_CONFIG_EH_MINVALUE_HYSTERESIS, ciet_CONFIG_EH_MINVALUE_CYCLEPERIOD,
																	ciet_CONFIG_EH_MINVALUE_suppliedVoltageCalc, ciet_CONFIG_EH_MINVALUE_suppliedCurrentCalc,
																	ciet_CONFIG_EH_MINVALUE_thCoupleVoltageCalc, ciet_CONFIG_EH_MINVALUE_thCoupleTemperatureCalc, 
																	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN]);
																	

	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[ciet_CONFIG_EH_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set aliases
	// set alias on only all DPEs below ProcessInput
//			cietConfigGenericDpFunctions_setAlias(sDpName, dsDpe, dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],exceptionInfo);
// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("EH", dsDpe, dsDescription);
	cietConfigGenericDpFunctions_setDescription(sDpName, dsDpe, dsDescription, exceptionInfo);

// set unit
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.suppliedVoltageCalc",".ProcessInput.Data.thCoupleVoltageCalc"), 
																								ciet_CONFIG_EH_FORMAT_VOLT, exceptionInfo);
																								
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.setPoint",".ProcessInput.Data.forcedSetPoint",
																															".ProcessInput.Data.powerCalc"), 
																								dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_UNIT], exceptionInfo);
																								
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.cyclePeriod"), 
																								ciet_CONFIG_EH_FORMAT_TIME, exceptionInfo);	

	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.threshold"), 
																								ciet_CONFIG_EH_FORMAT_THCOUPLE, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.suppliedCurrentCalc"), 
																								ciet_CONFIG_EH_FORMAT_A, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.thCoupleOffset",
																															".ProcessInput.Data.thCoupleTemperatureCalc"),																						
																								ciet_CONFIG_EH_FORMAT_THCOUPLE, exceptionInfo);																								
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.heaterResistance"), 
																								ciet_CONFIG_EH_FORMAT_OHM, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.thCoupleVoltageCalc"), 
																								ciet_CONFIG_EH_FORMAT_MILIVOLT, exceptionInfo);																								
																								
// set format
	cietConfigGenericDpFunctions_setFormat(sDpName,makeDynString(".ProcessInput.Data.heatVolMeas", 
																															".ProcessInput.Data.heatRefVolMeas",
																															".ProcessInput.Data.shuntVolMeas",
																															".ProcessInput.Data.shuntRefVolMeas",
																															".ProcessInput.Data.thVolMeas",
																															".ProcessInput.Data.thRefVolMeas")
																															,"#####", exceptionInfo);
	cietConfigGenericDpFunctions_setFormat(sDpName,makeDynString(".ProcessInput.Data.setPoint",
																									".ProcessInput.Data.forcedSetPoint",
																									".ProcessInput.Data.suppliedVoltageCalc",
																									".ProcessInput.Data.suppliedCurrentCalc",
																									 ".ProcessInput.Data.thCoupleVoltageCalc")
																									 ,"#####.##", exceptionInfo);
																									
  cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.threshold", "#####.###", exceptionInfo);	
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.heaterResistance", "#####.###", exceptionInfo);																										
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.thCoupleOffset", "#####.###", exceptionInfo);		
	cietConfigGenericDpFunctions_setFormat(sDpName,makeDynString(".ProcessInput.Data.powerCalc",".ProcessInput.Data.thCoupleTemperatureCalc"), "#####.###", exceptionInfo);																									
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.DeadBand.thCoupleTemperatureCalcDeadBand", "#####.###", exceptionInfo);		
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.DeadBand.powerCalcDeadBand", "#####.###", exceptionInfo);		
																									
// save the calibration curve name
		dpSet(sDpName+ ".ProcessInput.Curve.curveId",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_CALIBRATION] );
		
// set smoothing on all floats
/*	cietConfigGenericDpFunctions_setSmoothing(sDpName, dsConfigs, 
																					makeDynString(".ProcessInput.Data.heatVolMeas", 
																												".ProcessInput.Data.heatRefVolMeas",
																												".ProcessInput.Data.shuntVolMeas",
																												".ProcessInput.Data.shuntRefVolMeas",
																												".ProcessInput.Data.thVolMeas",
																												".ProcessInput.Data.thRefVolMeas",
																												".ProcessInput.Data.hysteresis",
																												".ProcessInput.Data.cyclePeriod", 
																												".ProcessInput.Data.suppliedVoltageCalc",
																												".ProcessInput.Data.suppliedCurrentCalc",
																												".ProcessInput.Data.thCoupleVoltageCalc",
																												".ProcessInput.Data.thCoupleTemperatureCalc",
																												".ProcessInput.Data.powerCalc"),
																					UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBANDTYPE,
																					makeDynInt(UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_heatVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_heatRefVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_shuntVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_shuntRefVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thRefVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_HYSTERESIS,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_CYCLEPERIOD,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_suppliedVoltageCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_suppliedCurrentCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thCoupleVoltageCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_thCoupleTemperatureCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_DEADBAND_powerCalc),
																					dfMin, dfMax, exceptionInfo);*/
	
// set pv_range on all floats
	cietConfigGenericDpFunctions_setPvRange(sDpName, 
																					makeDynString(".ProcessInput.Data.heatVolMeas", 
																												".ProcessInput.Data.heatRefVolMeas",
																												".ProcessInput.Data.shuntVolMeas",
																												".ProcessInput.Data.shuntRefVolMeas",
																												".ProcessInput.Data.thVolMeas",
																												".ProcessInput.Data.thRefVolMeas",
																												".ProcessInput.Data.hysteresis",
																												".ProcessInput.Data.cyclePeriod", 
																												".ProcessInput.Data.suppliedVoltageCalc",
																												".ProcessInput.Data.suppliedCurrentCalc",
																												".ProcessInput.Data.thCoupleVoltageCalc",
																												".ProcessInput.Data.thCoupleTemperatureCalc",																												
																												".ProcessInput.Data.powerCalc",
																												".ProcessInput.Data.setPoint",
																												".ProcessInput.Data.forcedSetPoint"),
																					dfMin, dfMax, exceptionInfo);

// set Range to visualize on plot
/*	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.powerCalc"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN], exceptionInfo);
																		 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.setPoint"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN], exceptionInfo);
*//*																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.forcedSetPoint"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_RANGEMIN], exceptionInfo);
																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.thCoupleTemperatureCalc"),
																			 		ciet_CONFIG_EH_MAXVALUE_thCoupleTemperatureCalc, ciet_CONFIG_EH_MINVALUE_thCoupleTemperatureCalc, exceptionInfo);										
*/																					
// set archiving 
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_ARCHIVEACTIVE], 
																				bArchive, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_ARCHIVETIMEFILTER],	
																				iArchiveType, iSmoothProcedure);
									
	cietConfigGenericFunctions_setArchiving("EH", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_ARCHIVETIMEFILTER],
																											exceptionInfo);

	sCMWDeviceName = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EH_CMWDEVICE];
	fwDevice_getAddressElements("EH", dsDpe, exceptionInfo, "");
	
	cietConfigGenericFunctions_setCMWAdress(sDpName, sCMWDeviceName, dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], dsDpe, exceptionInfo);	

}
//@}
	
