/**@name LIBRARY: cietConfig_PT.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 20 07 2005

Modification History: 
	
	21/02/07	Pablo   Changed curveId from description to Dp.	
	19/02/07	Pablo		Max and Min Range for Engineering values, taken from import file.
										Added format and unit for offset.	
	16/02/07	Pablo		Added Range Trend configuration.
	18/12/06	Pablo		Changed format resistanceCalc (#.####)
	17/11/06	Quique	Include format for pressureCalc (3D) and modify   resistanceCalc (4D)
	13/11/06	Quique	Include format resistanceCalc (#.###)
	21/08/06	Quique	Read the PT unit from specs file.
	
version 1.0

External Function : 
	. cietConfig_PT_setConfig :  set configuration of given DI datapoint
	. cietConfig_PT_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos DI configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//Constants



//@{
//------------------------------------------------------------------------------------------------------------------------

// cietConfigPT_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_PT_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "PT", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "PT", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigPT_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigPT_setConfig
/**
Purpose: set configuration of given PT datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_PT_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "PT", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "PT", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "PT", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_PT_checkConfig
/**
Purpose: check data before configuration for CRYOGTW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CRYOGTW_PT_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (ciet_CONFIG_PT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (ciet_CONFIG_PT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[ciet_CONFIG_PT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[ciet_CONFIG_PT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[ciet_CONFIG_PT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;

	if(bOk) {/*
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_senVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_refVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_gainValue], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_resistanceCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_pressureCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE], exceptionInfo);	*/																															
		// 4. Archive
		unConfigGenericFunctions_checkArchiveParameters(0, configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_PT_ARCHIVEACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CRYOGTW_PT_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_PT_setConfig
/**
Purpose:  set configuration of given PT datapoint for CRYOGTW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

CRYOGTW_PT_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sCMWDeviceName;
	dyn_string dsDpe, dsDescription;
	bool bArchive;
	int iArchiveTimeFilter, iArchiveType, iSmoothProcedure,i;
	
	// DebugN("FUNCTION: CRYOGTW_PT_setConfig(dsConfigs= "+dsConfigs);
	
	dyn_float dfMin = makeDynFloat(ciet_CONFIG_PT_MINVALUE_senVolMeas, ciet_CONFIG_PT_MINVALUE_refVolMeas,
																		ciet_CONFIG_PT_MINVALUE_gainValue, ciet_CONFIG_PT_MINVALUE_resistanceCalc,
																		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_RANGEMIN]);
	dyn_float dfMax = makeDynFloat(ciet_CONFIG_PT_MAXVALUE_senVolMeas, ciet_CONFIG_PT_MAXVALUE_refVolMeas,
																		ciet_CONFIG_PT_MAXVALUE_gainValue,ciet_CONFIG_PT_MAXVALUE_resistanceCalc,
		  															dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_RANGEMAX]);
			
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[ciet_CONFIG_PT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set aliases
	// set alias on only all DPEs below ProcessInput
	//cietConfigGenericDpFunctions_setAlias(sDpName, dsDpe, dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],exceptionInfo);

// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("PT", dsDpe, dsDescription);
	cietConfigGenericDpFunctions_setDescription(sDpName, dsDpe, dsDescription, exceptionInfo);
	
// set unit
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.senVolMeas", ".ProcessInput.Data.refVolMeas"), 
																				ciet_CONFIG_PT_UNIT_VOLT, exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.gainValue"), 
																				ciet_CONFIG_PT_UNIT_DB, exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.resistanceCalc"), 
																				ciet_CONFIG_PT_UNIT_OHMS, exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.offset"), 
																				ciet_CONFIG_PT_UNIT_OHMS, exceptionInfo);		
	// read from specs (importation file)		
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.pressureCalc"), 
																				dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_UNIT], exceptionInfo);
																						
	// cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.pressureCalc"), ciet_CONFIG_PT_UNIT_BAR, exceptionInfo);
																				
// save the calibration curve name
		dpSet(sDpName+ ".ProcessInput.Curve.curveId",
											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_CALIBRATION] );


// set format
	cietConfigGenericDpFunctions_setFormat(sDpName, 
																							makeDynString(".ProcessInput.Data.senVolMeas", 
																														".ProcessInput.Data.refVolMeas"), 
																							"######", exceptionInfo);
																						
  cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.gainValue", "#####", exceptionInfo);																								
  cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.pressureCalc", "#####.##", exceptionInfo);																								
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.resistanceCalc", "#####.###", exceptionInfo);
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.pressureCalc", "3D", exceptionInfo);
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.filterSamples", "#####", exceptionInfo);	
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.offset", "#####.###", exceptionInfo);		
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.DeadBand.resistanceCalcDeadBand", "#####.###", exceptionInfo);	

	



// set smoothing on all floats
/*
	cietConfigGenericDpFunctions_setSmoothing(sDpName, dsConfigs, 
																					makeDynString(".ProcessInput.Data.senVolMeas",
																												".ProcessInput.Data.refVolMeas",
																												".ProcessInput.Data.gainValue",
																												".ProcessInput.Data.resistanceCalc",
																												".ProcessInput.Data.pressureCalc"
																												),
																					UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBANDTYPE,
																					makeDynInt(UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_senVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_refVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_gainValue,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_resistanceCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_DEADBAND_pressureCalc
																										),
																					dfMin, dfMax, exceptionInfo);*/
	
// set pv_range on all floats
	cietConfigGenericDpFunctions_setPvRange(sDpName, 
																					makeDynString(".ProcessInput.Data.senVolMeas", 
																												".ProcessInput.Data.refVolMeas",
																												".ProcessInput.Data.gainValue",
																												".ProcessInput.Data.resistanceCalc",
																												".ProcessInput.Data.pressureCalc"),
																					dfMin, dfMax, exceptionInfo);
																					
// set Range to visualize on plot
/*	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.pressureCalc"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_RANGEMIN], exceptionInfo);
																		 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.resistanceCalc"),
																			 		ciet_CONFIG_PT_MAXVALUE_resistanceCalc, ciet_CONFIG_PT_MINVALUE_resistanceCalc, exceptionInfo);
																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.senVolMeas"),
																			 		ciet_CONFIG_PT_MAXVALUE_senVolMeas, ciet_CONFIG_PT_MINVALUE_senVolMeas, exceptionInfo);
																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.refVolMeas"),
																			 		ciet_CONFIG_PT_MAXVALUE_refVolMeas, ciet_CONFIG_PT_MINVALUE_refVolMeas, exceptionInfo);
*/

// set archiving 
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_ARCHIVEACTIVE], 
																				bArchive, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_ARCHIVETIMEFILTER],	
																				iArchiveType, iSmoothProcedure);
																				
	cietConfigGenericFunctions_setArchiving("PT", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_PT_ARCHIVETIMEFILTER],
																											exceptionInfo);

	
	
	sCMWDeviceName = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_PT_CMWDEVICE];
	// DebugN("==============================================");
	// DebugN("UN_CONFIG_ALLCOMMON_LENGTH= "+UN_CONFIG_ALLCOMMON_LENGTH);
	// DebugN("ciet_CONFIG_PT_CMWDEVICE= "+ciet_CONFIG_PT_CMWDEVICE);
	
	fwDevice_getAddressElements("PT", dsDpe, exceptionInfo, "");
	
	cietConfigGenericFunctions_setCMWAdress(sDpName, sCMWDeviceName, dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], dsDpe, exceptionInfo);	
	// DebugN("call cietConfigGenericFunctions_setCMWAdress()");
	// DebugN("sDpName= "+sDpName);
	// DebugN("sCMWDeviceName= "+sCMWDeviceName);
	// DebugN("dsConfigs["+UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM+"]= "+dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]);
	// DebugN("dsDpe= "+dsDpe);
	// DebugN("==============================================");
	
}

//------------------------------------------------------------------------------------------------------------------------
//@}

