/**@name LIBRARY: cietEHBS.ctl

@author: Alexey Merezhin (EN/ICE)

Creation Date: 16/10/2013

Modification History:
 16/10/13  amerezhi: init the object

version 1.0

External Function :
    . cietEHBS_FaceplateStatusDisconnection : animate faceplate disconnection
    . cietEHBS_ButtonUserAccess : User access for EHBS buttons
    . cietEHBS_ButtonSetState : set button state
    . cietEHBS_WidgetDisconnection : animate widget disconnection
    . cietEHBS_MenuConfiguration : configuration of popup menu

Internal Functions :
    . cietEHBS_FaceplateStatusRegisterCB : register callback function
    . cietEHBS_ButtonRegisterCB : register callback function
    . cietEHBS_ButtonAnimationCB : animate buttons
    . cietEHBS_ButtonDisconnection : animate buttons disconnection
    . cietEHBS_WidgetRegisterCB : register callback function
    . cietEHBS_WidgetAnimationCB : animate widget

    . cietEHBS_displayValueCyclePeriod : visualize right string in cyclePeriod

Purpose: This library contains EHBS object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
    . cietPT object (DPType etc.)
    . Global variables and $parameters defined in EHBS faceplate and widget
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/

/**
Purpose: Function called from the objectList to return the time and value

Parameters:
    - sDeviceName, string, input, device name
    - sDeviceType, string, input, unicos object
    - dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in Alarm faceplate
    . PVSS version: 3.1
    . operating system: WinXP
    . distributed system: yes.
*/
cietEHBS_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
    float value;
    string sTime;

    dpGet(sDeviceName+".ProcessInput.Data.powerCalc", value, sDeviceName+".ProcessInput.Data.powerCalc:_online.._stime", sTime);
    dsReturnData[1] = sTime;
    dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.powerCalc"), value) + " "
                            + dpGetUnit(sDeviceName+".ProcessInput.Data.powerCalc");
}

/**
Purpose: Displays the right string value using predified cyclePeriod values

Parameters:
    input:
        - iValue: value to visualize
        - sShape: place to visualize
        - sForeCol: color animation
        - bInvalid: validity
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: XP.
    . distributed system: yes.
*/
cietEHBS_displayValueCyclePeriod (int iValue, string sShape, string sForeCol, bool bInvalid)
{
    string formattedValue, sColor;

     if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_1_25s) formattedValue="1.25";
     else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_2_5s) formattedValue="2.5";
     else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_5s) formattedValue="5";
     else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_10s) formattedValue="10";

    sColor = sForeCol;
    if (bInvalid)
        {
        sColor = "unDataNotValid";
        }
    setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", formattedValue);
}

/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in cietPT faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/

cietEHBS_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
    int iAction, iRes, iAlertType1;
    dyn_string exceptionInfo;
    bool bRemote;

    string deviceName = unGenericDpFunctions_getDpName($sDpName);
    string sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    string sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    if (sPlcName != "") {
        if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
            g_bUnSystemAlarmPlc = true;
        }
        else {
            g_bUnSystemAlarmPlc = false;
        }
    }
    else
        g_bUnSystemAlarmPlc = false;

        unDistributedControl_isRemote(bRemote, sSystemName);
    if (bRemote)
        g_bSystemConnected = bSystemConnected;
    else
        g_bSystemConnected = true;

    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            cietEHBS_FaceplateStatusDisconnection();
            break;
        case UN_ACTION_DPCONNECT:

            g_bDisconnectCB = false;
            if (g_bUnSystemAlarmPlc) {
                iRes = dpConnect("cietEHBS_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

                g_bCallbackConnected = (iRes >= 0);
            }
            else
                cietEHBS_FaceplateStatusDisconnection();

            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:

            if (g_bUnSystemAlarmPlc)
                iRes = dpDisconnect("cietEHBS_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

            if (g_bDisconnectCB)
                cietEHBS_FaceplateDisconnect(deviceName, sPlcName);

            g_bCallbackConnected = !(iRes >= 0);
            g_bDisconnectCB = false;
                cietEHBS_FaceplateStatusDisconnection();

            break;
        case UN_ACTION_NOTHING:
        default:
            break;
    }
}

/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in PT widget
    . PVSS version: 2.12.1
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_FaceplateLockCallBack(string sdpe, bool bLockState)
{
    int iRes;
    string sPlcName;

    string deviceName = unGenericDpFunctions_getDpName(sdpe);
    if (bLockState) {
        if (g_bDisconnectCB) {
            cietEHBS_FaceplateDisconnect(deviceName, sPlcName);
            g_bDisconnectCB = false;
        }
        cietEHBS_FaceplateStatusDisconnection();
    }
    else {
        g_bDisconnectCB = true;
        cietEHBS_FaceplateConnect(deviceName);
    }
}

/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in cietPT faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_FaceplateConnect(string deviceName)
{
    int iRes;
    dyn_string exceptionInfo, dsFIPInfos, dsDeviceNameSplit;
    string sAliasAgent, sAgentName,sDpAgentName;

    string sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
    string sSystemName = unGenericDpFunctions_getSystemName(deviceName);

    if (sPlcName != "") {
        if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
            g_bUnSystemAlarmPlc = true;
        }
        else {
            g_bUnSystemAlarmPlc = false;
        }
    } else {
        g_bUnSystemAlarmPlc = false;
    }   

    g_sThRefVolMeasFormat       = dpGetFormat(deviceName + ".ProcessInput.Data.heatRelativeRefVol");
    g_sThRefVolMeasUnit             = dpGetUnit(deviceName + ".ProcessInput.Data.heatRelativeRefVol");
    g_sSuppliedVoltageCalcFormat        = dpGetFormat(deviceName + ".ProcessInput.Data.suppliedVoltageCalc");
    g_sSuppliedVoltageCalcUnit          = dpGetUnit(deviceName + ".ProcessInput.Data.suppliedVoltageCalc");
    g_sSuppliedCurrentCalcFormat        = dpGetFormat(deviceName + ".ProcessInput.Data.suppliedCurrentCalc");
    g_sSuppliedCurrentCalcUnit          = dpGetUnit(deviceName + ".ProcessInput.Data.suppliedCurrentCalc");
    g_sThCoupleVoltageCalcFormat        = dpGetFormat(deviceName + ".ProcessInput.Data.thermocoupleVoltageOrPtResistanceCalc");
    g_sThCoupleVoltageCalcUnit          = dpGetUnit(deviceName + ".ProcessInput.Data.thermocoupleVoltageOrPtResistanceCalc");
    g_sProtectionSensorOffsetFormat=dpGetFormat(deviceName + ".ProcessInput.Data.protectionSensorOffset");
    g_sProtectionSensorOffsetUnit=dpGetUnit(deviceName + ".ProcessInput.Data.protectionSensorOffset");
    g_sPowerCalcFormat      = dpGetFormat(deviceName + ".ProcessInput.Data.powerCalc");
    g_sPowerCalcUnit        = dpGetUnit(deviceName + ".ProcessInput.Data.powerCalc");
    g_sheaterResistanceFormat = dpGetFormat(deviceName + ".ProcessInput.Data.heaterResistance");
    g_sheaterResistanceUnit = dpGetUnit(deviceName + ".ProcessInput.Data.heaterResistance");
    g_sSetPointFormat   =dpGetFormat(deviceName + ".ProcessInput.Data.setPoint");
    g_sSetPointUnit         =dpGetUnit(deviceName + ".ProcessInput.Data.setPoint");
    g_sForcedSetPointFormat =dpGetFormat(deviceName + ".ProcessInput.Data.forcedSetPoint");
    g_sForcedSetPointUnit       =dpGetUnit(deviceName + ".ProcessInput.Data.forcedSetPoint");

    g_sThresholdFormat  =dpGetFormat(deviceName + ".ProcessInput.Data.threshold");
    g_sThresholdUnit    =dpGetUnit(deviceName + ".ProcessInput.Data.threshold");

    g_params["heatVolMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatVolMeas");
    g_params["heatVolMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatVolMeas");
    g_params["heatZeroVoltRefVolMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeas");
    g_params["heatZeroVoltRefVolMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeas");
    g_params["heatZeroVoltRefVolMeasInVoltsFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts");
    g_params["heatZeroVoltRefVolMeasInVoltsUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts");
    g_params["heatZeroHuitVoltRefVolMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeas");
    g_params["heatZeroHuitVoltRefVolMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeas");
    g_params["heatZeroHuitVoltRefVolMeasInVoltsFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts");
    g_params["heatZeroHuitVoltRefVolMeasInVoltsUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts");
    g_params["heatCurrentMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatCurrentMeas");
    g_params["heatCurrentMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.heatCurrentMeas");
    g_params["protectionVolMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.protectionVolMeas");
    g_params["protectionVolMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.protectionVolMeas");
    g_params["protectionRefVolMeasFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.protectionRefVolMeas");
    g_params["protectionRefVolMeasUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.protectionRefVolMeas");
    g_params["protectionVolMeasInVoltsFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.protectionVolMeasInVolts");
    g_params["protectionVolMeasInVoltsUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.protectionVolMeasInVolts");
    g_params["protectionRefVolMeasInVoltsFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.protectionRefVolMeasInVolts");
    g_params["protectionRefVolMeasInVoltsUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.protectionRefVolMeasInVolts");
    g_params["ThermistorVoltageFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.thermistorVoltageCalc");
    g_params["ThermistorVoltageUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.thermistorVoltageCalc");
    g_params["ThermistorTemperatureFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.thermistorTemperatureCalc");
    g_params["ThermistorTemperatureUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.thermistorTemperatureCalc");
    g_params["acSupplyVoltageFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.acSupplyVoltage");
    g_params["acSupplyVoltageUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.acSupplyVoltage");

    g_params["ThermocoupleVoltageFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.thermocoupleVoltageCalc");
    g_params["ThermocoupleVoltageUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.thermocoupleVoltageCalc");
    g_params["ThermocoupleTemperatureFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.thermocoupleTemperatureCalc");
    g_params["ThermocoupleTemperatureUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.thermocoupleTemperatureCalc");

    g_params["PTResistanceFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.ptResistanceCalc");
    g_params["PTResistanceUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.ptResistanceCalc");
    g_params["PTTemperatureFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.ptTemperatureCalc");
    g_params["PTTemperatureUnit"] = dpGetUnit(deviceName + ".ProcessInput.Data.ptTemperatureCalc");

    g_params["HeatCurrentACOffsetFormat"] = dpGetFormat(deviceName + ".ProcessInput.Data.heatCurrentACOffset");
    ptSensorRanges = cietConfig_EHBS_getPtSensorRanges();

    cietGenericFunctions_getFIPAgentInfos(deviceName, dsFIPInfos, exceptionInfo);

    if (dynlen(exceptionInfo) <= 0)
    {
        g_sGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];
        g_sAgentBus = dsFIPInfos[ciet_PARAMETER_BUS];
        g_sAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
        sAgentName =dsFIPInfos[ciet_PARAMETER_AGENTNAME];
        g_sAgentChannel = dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER];

        // Get Agent name to display also ioerror info
        g_sCompleteAgentName=unGenericDpFunctions_getWidgetDpName(sSystemName +sAgentName);

    }
    rawDatas.appendLines(3);
    rawDatas.sortOnClick(FALSE);

    if (g_bUnSystemAlarmPlc) {
        iRes = dpConnect("cietEHBS_FaceplateStatusAnimationCB",
                                            deviceName + ".ProcessInput.Config.sgNumber",
                                            deviceName + ".ProcessInput.Config.agNumber",
                                            deviceName + ".ProcessInput.Config.chNumber",
                                            deviceName + ".ProcessInput.Data.heatVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.heatRelativeRefVol",
                                            deviceName + ".ProcessInput.Data.cyclePeriod",
                                            deviceName + ".ProcessInput.Data.threshold",
                                            deviceName + ".ProcessInput.Data.setPoint",
                                            deviceName + ".ProcessInput.Data.forcedSetPoint",
                                            deviceName + ".ProcessInput.Data.sensorStatus",
                                            deviceName + ".ProcessInput.Data.sensorStatus2",
                                            g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
                                            deviceName + ".ProcessInput.Data.command",
                                            deviceName + ".ProcessInput.Data.suppliedVoltageCalc",
                                            deviceName + ".ProcessInput.Data.suppliedCurrentCalc",
                                            deviceName + ".ProcessInput.Data.powerCalc",
                                            deviceName + ".ProcessInput.Data.protectionSensorOffset",
                                            deviceName + ".ProcessInput.Data.heaterResistance",
                                            deviceName + ".ProcessInput.Data.thermocoupleVoltageCalc",
                                            deviceName + ".ProcessInput.Data.ptResistanceCalc",
                                            deviceName + ".ProcessInput.Data.thermocoupleTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.ptTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.thermistorTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.thermistorVoltageCalc",
                                            deviceName + ".ProcessInput.Data.thermistorTemperatureCalcWithMSBOnly",
                                            deviceName + ".ProcessInput.Data.heatCurrentACOffset",
                                            deviceName + ".ProcessInput.Data.ptSensorRange",
                                            deviceName + ".ProcessInput.Data.heatCurrentMeas",
                                            deviceName + ".ProcessInput.Data.protectionVolMeas",
                                            deviceName + ".ProcessInput.Data.protectionRefVolMeas",
                                            deviceName + ".ProcessInput.Data.protectionVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.protectionRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.acSupplyVoltage",
                                            deviceName + ".ProcessInput.Curve.curveId",
                                            deviceName + ".ProcessInput.Data.prodVariable1[1]",
                                            deviceName + ".ProcessInput.Data.prodVariable1[2]",
                                            deviceName + ".ProcessInput.Data.consVariable2[1]",
                                            deviceName + ".ProcessInput.Data.consVariable2[2]",
                                            deviceName + ".ProcessInput.Data.consVariable2[3]",
                                            deviceName + ".ProcessInput.Data.consVariable2[4]",
                                            deviceName + ".ProcessInput.Data.consVariable2[5]",
                                            deviceName + ".ProcessInput.Data.consVariable2[6]",
                                            deviceName + ".ProcessInput.Data.consVariable2[7]",
                                            deviceName + ".ProcessInput.Data.consVariable2[8]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[1]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[2]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[3]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[4]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[5]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[6]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[7]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[8]",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
                                            deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid");

    }
    else cietEHBS_FaceplateStatusDisconnection();
    g_bCallbackConnected = (iRes >= 0);
}

/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in cietPT faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_FaceplateDisconnect(string deviceName)
{
    int iRes;
    string sPlcName, sSystemName, sAlias, sAgentName, sDpAgentName;
    dyn_string dsDeviceNameSplit;

    if (g_bUnSystemAlarmPlc) {
        sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
        sSystemName = unGenericDpFunctions_getSystemName(deviceName);

        sAgentName = cietGenericFunctions_getAgentName(deviceName);
        sAlias = unGenericDpFunctions_getAlias(sAgentName);
        dsDeviceNameSplit=strsplit(sAlias, ".");

        iRes = dpDisconnect("cietEHBS_FaceplateStatusAnimationCB",
                                            deviceName + ".ProcessInput.Config.sgNumber",
                                            deviceName + ".ProcessInput.Config.agNumber",
                                            deviceName + ".ProcessInput.Config.chNumber",
                                            deviceName + ".ProcessInput.Data.heatVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeas",
                                            deviceName + ".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.heatRelativeRefVol",
                                            deviceName + ".ProcessInput.Data.cyclePeriod",
                                            deviceName + ".ProcessInput.Data.threshold",
                                            deviceName + ".ProcessInput.Data.setPoint",
                                            deviceName + ".ProcessInput.Data.forcedSetPoint",
                                            deviceName + ".ProcessInput.Data.sensorStatus",
                                            deviceName + ".ProcessInput.Data.sensorStatus2",
                                            g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
                                            deviceName + ".ProcessInput.Data.command",
                                            deviceName + ".ProcessInput.Data.suppliedVoltageCalc",
                                            deviceName + ".ProcessInput.Data.suppliedCurrentCalc",
                                            deviceName + ".ProcessInput.Data.powerCalc",
                                            deviceName + ".ProcessInput.Data.protectionSensorOffset",
                                            deviceName + ".ProcessInput.Data.heaterResistance",
                                            deviceName + ".ProcessInput.Data.thermocoupleVoltageCalc",
                                            deviceName + ".ProcessInput.Data.ptResistanceCalc",
                                            deviceName + ".ProcessInput.Data.thermocoupleTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.ptTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.thermistorTemperatureCalc",
                                            deviceName + ".ProcessInput.Data.thermistorVoltageCalc",
                                            deviceName + ".ProcessInput.Data.thermistorTemperatureCalcWithMSBOnly",
                                            deviceName + ".ProcessInput.Data.heatCurrentACOffset",
                                            deviceName + ".ProcessInput.Data.ptSensorRange",
                                            deviceName + ".ProcessInput.Data.heatCurrentMeas",
                                            deviceName + ".ProcessInput.Data.protectionVolMeas",
                                            deviceName + ".ProcessInput.Data.protectionRefVolMeas",
                                            deviceName + ".ProcessInput.Data.protectionVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.protectionRefVolMeasInVolts",
                                            deviceName + ".ProcessInput.Data.acSupplyVoltage",
                                            deviceName + ".ProcessInput.Curve.curveId",
                                            deviceName + ".ProcessInput.Data.prodVariable1[1]",
                                            deviceName + ".ProcessInput.Data.prodVariable1[2]",
                                            deviceName + ".ProcessInput.Data.consVariable2[1]",
                                            deviceName + ".ProcessInput.Data.consVariable2[2]",
                                            deviceName + ".ProcessInput.Data.consVariable2[3]",
                                            deviceName + ".ProcessInput.Data.consVariable2[4]",
                                            deviceName + ".ProcessInput.Data.consVariable2[5]",
                                            deviceName + ".ProcessInput.Data.consVariable2[6]",
                                            deviceName + ".ProcessInput.Data.consVariable2[7]",
                                            deviceName + ".ProcessInput.Data.consVariable2[8]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[1]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[2]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[3]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[4]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[5]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[6]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[7]",
                                            deviceName + ".ProcessInput.Data.prodVariable3[8]",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
                                            deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid");
        }
    g_bCallbackConnected = !(iRes >= 0);
}

/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables defined in cietPT faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_FaceplateStatusAnimationCB(string sDp_sgNumber, int isgNumber,
                                    string sDp_agNumber, int iagNumber,
                                    string sDp_chNumber, int ichNumber,
                                    string sDp_heatVolMeas, float fheatVolMeas,
                                    string sDp_heatZeroVoltRefVolMeas, float fheatZeroVoltRefVolMeas,
                                    string sDp_heatZeroVoltRefVolMeasInVolts, float fheatZeroVoltRefVolMeasInVolts,
                                    string sDp_heatZeroHuitVoltRefVolMeas, float fheatZeroHuitVoltRefVolMeas,
                                    string sDp_heatZeroHuitVoltRefVolMeasInVolts, float fheatZeroHuitVoltRefVolMeasInVolts,
                                    string sDp_heatRelativeRefVol, float fheatRelativeRefVol,
                                    string sDp_cyclePeriod, int icyclePeriod,
                                    string sDp_threshold, float fThreshold,
                                    string sDp_setPoint, float fSetPoint,
                                    string sDp_forcedSetPoint, float fforcedSetPoint,
                                    string sDp_sensorStatus, unsigned uSensorStatus,
                                    string sDp_sensorStatus2, unsigned uSensorStatus2,
                                    string sDp_AgentStatus, unsigned uAgentStatus,
                                    string sDp_commandIn, unsigned uCommandIn,
                                    string sDp_suppliedVoltageCalc, float fsuppliedVoltageCalc,
                                    string sDp_suppliedCurrentCalc, float fsuppliedCurrentCalc,
                                    string sDp_powerCalc, float fpowerCalc,
                                    string sDp_protectionSensorOffset, float fprotectionSensorOffset,
                                    string sDp_thheaterResistance, float fheaterResistance,
                                    string sDp_thermocoupleVoltageCalc, float thermocoupleVoltageCalc,
                                    string sDp_ptResistanceCalc, float ptResistanceCalc,
                                    string sDp_thermocoupleTemperatureCalc, float thermocoupleTemperatureCalc,
                                    string sDp_ptTemperatureCalc, float ptTemperatureCalc,
                                    string sDp_thermistorTemperatureCalc, float thermistorTemperatureCalc,
                                    string sDp_thermistorVoltageCalc, float thermistorVoltageCalc,
                                    string sDp_thermistorTemperatureCalcWithMSBOnly, float thermistorTemperatureCalcWithMSBOnly,
                                    string sDp_heatCurrentACOffset, float heatCurrentACOffset,
                                    string sDp_ptSensorRange, int ptSensorRange,
                                    string sDp_heatCurrentMeas, float heatCurrentMeas,
                                    string sDp_protectionVolMeas, float protectionVolMeas,
                                    string sDp_protectionRefVolMeas, float protectionRefVolMeas,
                                    string sDp_protectionVolMeasInVolts, float protectionVolMeasInVolts,
                                    string sDp_protectionRefVolMeasInVolts, float protectionRefVolMeasInVolts,
                                    string sDp_avSupplyVoltage, float acSupplyVoltageValue,
                                    string sDp_curveId, string sCurveId,
                                    string sDp_prodVariable1_1, bit32 bprodVariable1_1,
                                    string sDp_prodVariable1_2, bit32 bprodVariable1_2,
                                    string sDp_consVariable2_1, bit32 bconsVariable2_1,
                                    string sDp_consVariable2_2, bit32 bconsVariable2_2,
                                    string sDp_consVariable2_3, bit32 bconsVariable2_3,
                                    string sDp_consVariable2_4, bit32 bconsVariable2_4,
                                    string sDp_consVariable2_5, bit32 bconsVariable2_5,
                                    string sDp_consVariable2_6, bit32 bconsVariable2_6,
                                    string sDp_consVariable2_7, bit32 bconsVariable2_7,
                                    string sDp_consVariable2_8, bit32 bconsVariable2_8,
                                    string sDp_prodVariable3_1, bit32 bprodVariable3_1,
                                    string sDp_prodVariable3_2, bit32 bprodVariable3_2,
                                    string sDp_prodVariable3_3, bit32 bprodVariable3_3,
                                    string sDp_prodVariable3_4, bit32 bprodVariable3_4,
                                    string sDp_prodVariable3_5, bit32 bprodVariable3_5,
                                    string sDp_prodVariable3_6, bit32 bprodVariable3_6,
                                    string sDp_prodVariable3_7, bit32 bprodVariable3_7,
                                    string sDp_prodVariable3_8, bit32 bprodVariable3_8,
                                    string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue,
                                    string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
                                    string sDpInvalid, bool bCodeInvalid)
{

    int iAlarmValue,i,j, iStart;
    string sTemp,sDpPLCName, sOperationMode, sBusName, sCrate;
    dyn_bit32 db32sensorByte;

    dyn_bit32 db32sensorByte_1= makeDynChar(bprodVariable1_1,bprodVariable1_2);
    dyn_bit32 db32sensorByte_2= makeDynChar(bconsVariable2_1,bconsVariable2_2,bconsVariable2_3,bconsVariable2_4,
                                                                                bconsVariable2_5,bconsVariable2_6,bconsVariable2_7,bconsVariable2_8);
    dyn_bit32 db32sensorByte_3= makeDynChar(bprodVariable3_1,bprodVariable3_2,bprodVariable3_3,bprodVariable3_4,bprodVariable3_5,bprodVariable3_6,bprodVariable3_7,bprodVariable3_8);

    dyn_dyn_anytype dsVariable =    makeDynAnytype(db32sensorByte_1, db32sensorByte_2, db32sensorByte_3);

    float fheatRefVolValue,fheatVolValue, fshuntRefVolValue,fshuntVolValue, fthRefVolValue, fthVolValue;

    bool bFaceplateInvalid = (bCodeInvalid || !bSystemIntegrityAlarmEnabled ||  (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value));
    bool bForcedProductionRawData = getBit(uSensorStatus, 13);
    bool thermocoupleMode = getBit(uSensorStatus2, 6) == 0;

    sCrate= cietGenericObject_getCrate ($sDpName);
    if (g_bSystemConnected)
        cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", bFaceplateInvalid);

    if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))
            iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
    else
        iAlarmValue=c_unSystemIntegrity_no_alarm_value;

    iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
    sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
    cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled /*&& bSystemAlarmEnabled*/, bCodeInvalid);

    //  --- Configuration  ---
    if (getBit(uSensorStatus, 5)) sOperationMode = "AC";
    else sOperationMode = "DC";

    g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);

    //  --- Warnings   ---
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("AG_IOERROR", !getBit(uAgentStatus,0), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("DISABLE", !getBit(uSensorStatus, 12), 1, "",FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("IO_ERROR", !getBit(uSensorStatus, 0), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("RESET", !getBit(uSensorStatus, 1), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("BADCONFIG", !getBit(uSensorStatus, 2), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("FORCED_MODE", !getBit(uSensorStatus, 3), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("FORCED_MODE_PRODUCTION_RAWDATA", !getBit(uSensorStatus, 13), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("forcedModeEnabled", !getBit(uSensorStatus2, ciet_EHBS_ENABLE_DISABLE_FORCED_BIT), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("forcedModeOn", !getBit(uSensorStatus2, ciet_EHBS_ON_OFF_FORCED_BIT), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("cardProtection", !getBit(uSensorStatus, ciet_EHBS_CARD_PROTECTION_BIT), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("cardOverTemp", !getBit(uSensorStatus2, ciet_EHBS_CARD_OVER_TEMP_BIT), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("heaterOverTemp", !getBit(uSensorStatus, ciet_EHBS_HEATER_OVER_TEMP_BIT), 1, "", FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("watchdog", !getBit(uSensorStatus2, ciet_EHBS_WATCHDOG_BIT), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("powerCycle", !getBit(uSensorStatus2, ciet_EHBS_POWER_CYCLE_BIT), 1, "", FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("thermisterTempAboveMid", !getBit(uSensorStatus2, 11), 1, "",FALSE, "cietFaceplate_Warning");
    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("hardwareError", !getBit(uSensorStatus, 14), 1, "",FALSE, "cietFaceplate_Warning");

    //  --- Measurements  ---
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatVolMeasFormat"], g_params["heatVolMeasUnit"], fheatVolMeas, "heatVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatZeroVoltRefVolMeasFormat"], g_params["heatZeroVoltRefVolMeasUnit"], fheatZeroVoltRefVolMeas, "heatZeroVoltRefVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatZeroVoltRefVolMeasInVoltsFormat"], g_params["heatZeroVoltRefVolMeasInVoltsUnit"], fheatZeroVoltRefVolMeasInVolts, "heatZeroVoltRefVolMeasInVolts", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatZeroHuitVoltRefVolMeasFormat"], g_params["heatZeroHuitVoltRefVolMeasUnit"], fheatZeroHuitVoltRefVolMeas, "heatZeroHuitVoltRefVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatZeroHuitVoltRefVolMeasInVoltsFormat"], g_params["heatZeroHuitVoltRefVolMeasInVoltsUnit"], fheatZeroHuitVoltRefVolMeasInVolts, "heatZeroHuitVoltRefVolMeasInVolts", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["heatCurrentMeasFormat"], g_params["heatCurrentMeasUnit"], heatCurrentMeas, "heatCurrentMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["protectionVolMeasFormat"], g_params["protectionVolMeasUnit"], protectionVolMeas, "protectionVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["protectionRefVolMeasFormat"], g_params["protectionRefVolMeasUnit"], protectionRefVolMeas, "protectionRefVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["protectionVolMeasInVoltsFormat"], g_params["protectionVolMeasInVoltsUnit"], protectionVolMeasInVolts, "protectionVolMeasInVolts", "unDisplayValue_Status", bFaceplateInvalid);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["protectionRefVolMeasInVoltsFormat"], g_params["protectionRefVolMeasInVoltsUnit"], protectionRefVolMeasInVolts, "protectionRefVolMeasInVolts", "unDisplayValue_Status", bFaceplateInvalid);


    //  --- Constants  ---
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["acSupplyVoltageFormat"], g_params["acSupplyVoltageUnit"], acSupplyVoltageValue, "acSupplyVoltage", "unDisplayValue_Parameter", bFaceplateInvalid);
    if (g_bSystemConnected)
        cietEHBS_displayValueCyclePeriod(icyclePeriod,"CYCLEPERIOD", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_sThresholdFormat, g_sThresholdUnit, fThreshold, "THRESHOLD", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_sProtectionSensorOffsetFormat, g_sProtectionSensorOffsetUnit, fprotectionSensorOffset, "PROTECTIONSENSOROFFSET", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_sheaterResistanceFormat, g_sheaterResistanceUnit, fheaterResistance, "HEATERRESISTANCE", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["HeatCurrentACOffsetFormat"], "", heatCurrentACOffset, "heatCurrentACOffset", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected) {
        if (mappingHasKey(ptSensorRanges, ptSensorRange)) {
            cietGenericObject_DisplayString(ptSensorRanges[ptSensorRange], "ptSensorRange", "unDisplayValue_Parameter", FALSE);
        } else {
            cietGenericObject_DisplayString(ptSensorRange, "ptSensorRange", "unDisplayValue_Parameter", FALSE);
        }
    }

    // --- Calculations  ---

    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_sSuppliedVoltageCalcFormat, g_sSuppliedVoltageCalcUnit, fsuppliedVoltageCalc, "SUPPLIEDVOLTAGECALC", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_sSuppliedCurrentCalcFormat, g_sSuppliedCurrentCalcUnit, fsuppliedCurrentCalc, "SUPPLIEDCURRENTCALC", "unDisplayValue_Parameter", FALSE);

    if (g_bSystemConnected) {
        unGenericObject_DisplayValue(g_sPowerCalcFormat, g_sPowerCalcUnit, fpowerCalc, "POWERCALC", "unDisplayValue_Parameter", FALSE);
    }
    if (g_bSystemConnected) {
        if (!bForcedProductionRawData) {
            unGenericObject_DisplayValue(g_sSetPointFormat, g_sSetPointUnit, fSetPoint, "SETPOINT", "unDisplayValue_Parameter", FALSE);
        } else {
            cietGenericObject_DisplayString("Forced RawData", "SETPOINT", "unDisplayValue_Parameter", FALSE);
        }
    }
    if (g_bSystemConnected) {
        if (!bForcedProductionRawData) {
            unGenericObject_DisplayValue(g_sForcedSetPointFormat, g_sForcedSetPointUnit, fforcedSetPoint, "FORCEDSETPOINT", "unDisplayValue_Parameter", FALSE);
        } else {
            cietGenericObject_DisplayString("Forced RawData", "FORCEDSETPOINT", "unDisplayValue_Parameter", FALSE);
        }
    }

    if (thermocoupleMode) {
        setMultiValue("ThOrPtVoltage", "text", "ThermoC voltage", "ThOrPtTemp", "text", "ThermoC temperature");
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["ThermocoupleVoltageFormat"], g_params["ThermocoupleVoltageUnit"], thermocoupleVoltageCalc, "THCOUPLEVOLTAGECALC", "unDisplayValue_Parameter", FALSE);
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["ThermocoupleTemperatureFormat"], g_params["ThermocoupleTemperatureUnit"], thermocoupleTemperatureCalc, "THCOUPLETEMPERATURECALC", "unDisplayValue_Parameter", FALSE);
    } else {
        setMultiValue("ThOrPtVoltage", "text", "PT resistance", "ThOrPtTemp", "text", "PT temperature");
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["PTResistanceFormat"], g_params["PTResistanceUnit"], ptResistanceCalc, "THCOUPLEVOLTAGECALC", "unDisplayValue_Parameter", FALSE);
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["PTTemperatureFormat"], g_params["PTTemperatureUnit"], ptTemperatureCalc, "THCOUPLETEMPERATURECALC", "unDisplayValue_Parameter", FALSE);
    }

    if (getBit(uSensorStatus2, ciet_EHBS_SENSOR_STATUS2_THERMISTOR_MODE)) {
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["ThermistorVoltageFormat"], g_params["ThermistorVoltageUnit"], thermistorVoltageCalc, "thermisterVoltage", "unDisplayValue_Parameter", FALSE);
        if (g_bSystemConnected)
            unGenericObject_DisplayValue(g_params["ThermistorTemperatureFormat"], g_params["ThermistorTemperatureUnit"], thermistorTemperatureCalc, "thermisterTemp", "unDisplayValue_Parameter", FALSE);
    } else {
        if (g_bSystemConnected)
            cietGenericObject_DisplayString("No calc", "thermisterVoltage", "unDisplayValue_Parameter", FALSE);
        if (g_bSystemConnected)
            cietGenericObject_DisplayString("No calc", "thermisterTemp", "unDisplayValue_Parameter", FALSE);
    }
    if (g_bSystemConnected)
        unGenericObject_DisplayValue(g_params["ThermistorTemperatureFormat"], g_params["ThermistorTemperatureUnit"], thermistorTemperatureCalcWithMSBOnly, "thermisterMidVoltage", "unDisplayValue_Parameter", FALSE);

    //  --- Configuration ---
    if (g_bSystemConnected) {
        if (!bForcedProductionRawData) {
            cietGenericObject_DisplayString(sOperationMode, "MODE_AC_DC", "unDisplayValue_Parameter", FALSE);
        } else {
            cietGenericObject_DisplayString("Forced RawData", "MODE_AC_DC", "unDisplayValue_Parameter", FALSE);
        }
    }

    if (g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("thermistorMode", !getBit(uSensorStatus2, ciet_EHBS_SENSOR_STATUS2_THERMISTOR_MODE), 1, "",FALSE, "cietFaceplate_ModeActive");
    if (g_bSystemConnected)
        cietGenericObject_DisplayString(thermocoupleMode ? "thermocouple" : "PT", "deviceMode", "unDisplayValue_Parameter", FALSE);


// --- WFIP Infos  ---
    if (g_bSystemConnected)
        cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected)
        cietGenericObject_DisplayInt(ichNumber, "AGENTCHANNEL", "unDisplayValue_Parameter", FALSE);
    if (g_bSystemConnected) {
        sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
        // DebugN("sDp_sgNumber="+sDp_sgNumber);
        // DebugN("sBusName="+sBusName);
        cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter", FALSE);
        cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", FALSE);
    }

    //  --- RawData  ---

    // case 1st row, place the two bytes in the right position
    db32sensorByte=dsVariable[1];

    for (i=0;i<=6;i=i+2) {
        j=i+1;

        if ((i==2*ichNumber) && (g_bSystemConnected)) {
            cietGenericObject_DisplayToCell(cietGenericObject_toHexa(db32sensorByte[1]), "rawDatas",0, "B"+i, "unDisplayValue_Parameter", FALSE);
            cietGenericObject_DisplayToCell(cietGenericObject_toHexa(db32sensorByte[2]), "rawDatas",0, "B"+j, "unDisplayValue_Parameter", FALSE);
        } else {
            cietGenericObject_DisplayToCell("--", "rawDatas",0, "B"+i, "unDisplayValue_Parameter", FALSE);
            cietGenericObject_DisplayToCell("--", "rawDatas",0, "B"+j, "unDisplayValue_Parameter", FALSE);
        }
    }

    // for the last two rows
    for(int k=2;k<=dynlen(dsVariable);k++) {
        db32sensorByte = dsVariable[k];
        for(i=1;i<=8;i++) {
            int j =i-1;

            if ((k==3) && (i<=4)) {
                if (j!=ichNumber)
                    rawDatas.cellBackColRC(2,"B"+j,"_3DFace");
                else
                    rawDatas.cellBackColRC(2,"B"+j,"_Window");
            }
            sTemp = cietGenericObject_toHexa(db32sensorByte[i]);
            if (g_bSystemConnected)
                cietGenericObject_DisplayToCell(sTemp, "rawDatas",k-1, "B"+j, "unDisplayValue_Parameter", FALSE);
        }
    }

    // Not relevant to the information
    rawDatas.cellBackColRC(2,"B5","_3DFace");
    rawDatas.cellBackColRC(2,"B6","_3DFace");
    rawDatas.cellBackColRC(2,"B7","_3DFace");


    //  --- Calibration Curve  ---
    cietGenericObject_DisplayString(sCurveId, "idCurve", "unDisplayValue_Parameter", FALSE);
}

/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_FaceplateStatusDisconnection()
{
    //unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "AG_IOERROR", /*"OPERATION_AC","OPERATION_DC",*/ "FORCED_MODE" ));

    /*unGenericObject_DisplayValueDisconnect(makeDynString("MODE_AC_DC", "GTWNAME", "AGENTNUMBER", "AGENTBUS", "AGENTCHANNEL", "HEATVOLMEAS",
    "HEATREFVOLMEAS", "SHUNTVOLMEAS", "SHUNTREFVOLMEAS", "THVOLMEAS", "THREFVOLMEAS", "HYSTERESIS", "CYCLEPERIOD",  "THRESHOLD",
    "SETPOINT", "FORCEDSETPOINT", "SENSORSTATUS",   "COMMANDIN","SUPPLIEDVOLTAGECALC", "SUPPLIEDCURRENTCALC","THCOUPLEVOLTAGECALC",*//*"THCOUPLETEMPERATURECALC",*//*"POWERCALC", "PROTECTIONSENSOROFFSET",
    "HEATERRESISTANCE", "B11_sensorByte","B10_sensorByte","B9_sensorByte","B8_sensorByte","B7_sensorByte", "B6_sensorByte", "B5_sensorByte",
    "B4_sensorByte", "B3_sensorByte", "B2_sensorByte", "B1_sensorByte", "B0_sensorByte"));*/

  //put in the top textbox the DATA not connected in purple
  cietGenericFunctions_HeaderDisconnect();
}

/**
Purpose: User access for EHBS buttons

Parameters:
    - sDpName, string, input, device name
    - sDpType, string, input, dptype
    - dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
    bool operator, expert, admin;
    int i, length;
    dyn_bool dbPermissions;
    dyn_string exceptionInfo;
    string sOperAction, sExpAction, sAdminAction;
    bool bActionDefined = false;

    unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
    if (sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME || sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME || sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME) {
        bActionDefined=true;
    }
    operator = dbPermissions[2];
    expert = dbPermissions[3];
    admin = dbPermissions[4];
    dsAccess = makeDynString();
    if (operator) {
          if (bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT,
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
    }
    if (expert) {
        if (bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
        else
            dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,
                                              UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,
                                              ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT,
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG,
                                              ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT,
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE,
                                              ciet_FACEPLATE_BUTTON_ACTIVATION, ciet_FACEPLATE_BUTTON_DIAGNOSTIC));
    }
    if (admin) {
        if (bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        else
            dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,
                                              UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,
                                              ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT,
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG,
                                              ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT,
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
    }

}

/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in DQAMCMB faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
    string deviceName, sSystemName, sDpType, sFunction;
    int iAction, iRes, iManReg01;
    dyn_string exceptionInfo, dsFunctions, dsUserAccess;

    deviceName = unGenericDpFunctions_getDpName($sDpName);
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);

    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction)
        {
        case UN_ACTION_DISCONNECT:
            cietEHBS_ButtonDisconnection();
            break;
        case UN_ACTION_DPCONNECT:
            sDpType="EHBS";
            unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
            sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];

            if (sFunction != "") {
                evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
                             "dyn_string dsUserAccessReturn;" +
                             "if (isFunctionDefined(\"" + sFunction  + "\"))" +
                             "    {" +
                                sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
                             "    }" +
                             "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);

                if (dynlen(dsUserAccess) == 0)
                    fwException_raise(exceptionInfo,"ERROR", "cietEHBS_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
                else
                    g_dsUserAccess=dsUserAccess;
            } else {
                fwException_raise(exceptionInfo,"ERROR", "cietEHBS_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
            }


            iRes = dpConnect("cietEHBS_ButtonAnimationCB",
                                             deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                             deviceName + ".statusInformation.selectedManager",
                                             deviceName + ".ProcessInput.Data.sensorStatus");
            g_bCallbackConnected = (iRes >= 0);

            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            iRes = dpDisconnect("cietEHBS_ButtonAnimationCB",
                                                    deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                                    deviceName + ".statusInformation.selectedManager",
                                                    deviceName + ".ProcessInput.Data.sensorStatus");
            g_bCallbackConnected = !(iRes >= 0);
            cietEHBS_ButtonDisconnection();
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
    }
}

/**
Purpose: animate buttons

Parameters:
    - dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
    - dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
    . This library works with an unicos button panel and need global variables declared in this panel
        -> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_ButtonAnimationCB(string sDpLock, bool bLocked,
                             string sDpSelect, string sSelectedManager,
                             string sDpSensorStatus, unsigned uSensorStatus)
{
    bool bForcedMode, bStatusRawData, bDisable;
    string sDisableText;

    bDisable = getBit(uSensorStatus, 12);

    if (bDisable == 1)
        sDisableText=ciet_FACEPLATE_BUTTON_ENABLE;
    else if (bDisable ==0)
        sDisableText=ciet_FACEPLATE_BUTTON_DISABLE;

    setMultiValue("DISABLE_BUTTON.buttonDisable", "text", sDisableText);

    if (StatusRawDataText.text=="true")
        bStatusRawData = 1;
    else if (StatusRawDataText.text=="false")
        bStatusRawData = 0;


    bForcedMode = getBit(uSensorStatus,3);

    cietEHBS_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "EHBS", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bForcedMode,bStatusRawData));
}

// cietEHBS_ButtonSetState
/**
Purpose: set button state

Parameters:
    - deviceName, string, input, device
    - sDpType, string, input, dptype
    - dsUserAccess, dyn_string, input, user access
    - dsData, dyn_string, parameter

Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
    bool bSelected, buttonEnabled, bit1, bit2, bForcedMode, bStatusRawData;
    dyn_string dsButtons;
    int i, length, alertType;
    string localManager;
    bool bLocked, ListEnabled;
    string sSelectedManager;
    unsigned uSensorStatus;

    bLocked = dsData[1];
    sSelectedManager = dsData[2];
    bForcedMode = dsData[3];
    bStatusRawData = dsData[4];


    // 1. Selection state
    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
    bSelected = (localManager == "S");

    dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS, ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_AUTO_MODE,ciet_FACEPLATE_BUTTON_SETPOINT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ACTIVATION, ciet_FACEPLATE_BUTTON_DIAGNOSTIC); // Each button except "Select"
    length = dynlen(dsButtons);

    // 2. Button state

    for(i=1;i<=length;i++) {
        buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
        switch (dsButtons[i]) {
            case ciet_FACEPLATE_BUTTON_RESET:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_CONSTANTS:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_CALIBRATION:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_DEADBANDS:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_CONFIG:
                buttonEnabled = buttonEnabled && bSelected;
                break;
            case ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA:
                buttonEnabled = buttonEnabled && bSelected;
                break;
            case ciet_FACEPLATE_BUTTON_RAWDATA:
                buttonEnabled = buttonEnabled && bSelected && !bStatusRawData;
            //  DebugN ("Button Status = " + bStatusRawData );
                break;

            case ciet_FACEPLATE_BUTTON_FORCED_MODE:
                buttonEnabled = buttonEnabled && bSelected && !bForcedMode;
                break;

            case ciet_FACEPLATE_BUTTON_AUTO_MODE:
                buttonEnabled = buttonEnabled && bSelected && bForcedMode;
                break;

            case ciet_FACEPLATE_BUTTON_SETPOINT:
                buttonEnabled = buttonEnabled && bSelected && bForcedMode;
                break;

            case ciet_FACEPLATE_BUTTON_TENDPARAM:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_DEBUG:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_DISABLE:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_ACTIVATION:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            case ciet_FACEPLATE_BUTTON_DIAGNOSTIC:
                buttonEnabled = buttonEnabled && bSelected;
                break;

            default:
                break;
        }
        setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
    }
    // 3. Button Select state
    unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

/**
Purpose: function called before the button set state when a user logs in.

Parameters:
    deviceName: string, input, the device name
    dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
    . This library works with an unicos button panel and need global variables declared in this panel
        -> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
        -> $sDpName, string, the device DP name
        -> g_sFaceplateButtonType, string, faceplate type
    . PVSS version: 3.0
    . operating system: NT, W2000 and Linux, but tested only under W2000.
    . distributed system: yes but not tested.
*/
cietEHBS_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData) {
    int iRes;
    string sDpLock, sDpSelectedManager, sSelectedManager, sSensorStatusDp;
    bool bLocked;
    unsigned uSensorStatus;

    string sSystemName = unGenericDpFunctions_getSystemName(deviceName);

    sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
    sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
    sSensorStatusDp = deviceName + ".ProcessInput.Data.sensorStatus";

    if (dpExists(deviceName)) {
        iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager, sSensorStatusDp,uSensorStatus);
    } else {
        iRes = -1;
    }
    if (iRes < 0) { // Errors during dpget
        sDpLock = "";
        bLocked = false;
        sDpSelectedManager = "";
        sSelectedManager = "";
        sInvalidDp = "";
        bInvalid = false;
    }
    dsReturnData=makeDynString(bLocked, sSelectedManager, getBit(uSensorStatus,3), "", "");
}


/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_ButtonDisconnection()
{
    unGenericObject_ButtonDisconnect(makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA, ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_DEADBANDS, ciet_FACEPLATE_BUTTON_CALIBRATION, ciet_FACEPLATE_BUTTON_CONSTANTS, ciet_FACEPLATE_BUTTON_FORCED_MODE, ciet_FACEPLATE_BUTTON_AUTO_MODE, ciet_FACEPLATE_BUTTON_SETPOINT,UN_FACEPLATE_BUTTON_SELECT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE));
}

/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in PT widget
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
    string deviceName, deviceType;
    int iAction,iRes;
    dyn_string exceptionInfo;
    string sFrontEnd;
    bool bRemote;

    deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

    g_bUnSystemAlarmPlc = false;
    string sSystemName = unGenericDpFunctions_getSystemName(deviceName);
    string sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

    unDistributedControl_isRemote(bRemote, sSystemName);
    if (bRemote)
        g_bSystemConnected = bSystemConnected;
    else
        g_bSystemConnected = true;


    if (sPlcName != "") {
        if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
            g_bUnSystemAlarmPlc = true;
        } else {
            g_bUnSystemAlarmPlc = false;
        }
    }

    unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
    switch (iAction) {
        case UN_ACTION_DISCONNECT:
            cietEHBS_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_DPCONNECT:
            g_sDpName = deviceName;
            g_bDisconnectCB = false;

            if (g_bUnSystemAlarmPlc) {
                deviceType = dpTypeName(deviceName);
                if (deviceType == "EHBS") {
                    iRes = dpConnect("cietEHBS_WidgetLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");
                } else {
                    g_bUnSystemAlarmPlc = false;
                    cietEHBS_WidgetDisconnection(g_sWidgetType);
                }

                g_bCallbackConnected = (iRes >= 0);
            } else {
                cietEHBS_WidgetDisconnection(g_sWidgetType);
            }
            break;
        case UN_ACTION_DPDISCONNECT_DISCONNECT:
            if (g_bUnSystemAlarmPlc)
                iRes = dpDisconnect("cietEHBS_WidgetLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

            if (g_bDisconnectCB) {
                cietEHBS_WidgetDisconnect(g_sDpName, sPlcName);
            }
            g_bCallbackConnected = !(iRes >= 0);
            g_bDisconnectCB = false;

            cietEHBS_WidgetDisconnection(g_sWidgetType);
            break;
        case UN_ACTION_NOTHING:
        default:
            break;
        }
}

/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables and $parameters defined in PT widget
    . PVSS version: 2.12.1
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_WidgetLockCallBack(string sdpe, bool bLockState)
{
    int iRes;
    string sPlcName;

    if (bLockState)
        { // lock set so disconnect if not already disconnected
        if (g_bDisconnectCB)
            {
                cietEHBS_WidgetDisconnect(g_sDpName, iRes);
            }
        g_bDisconnectCB = false;
        cietEHBS_WidgetDisconnection(g_sWidgetType);
        }
    else
        { // lock not set so connect if not already connected
        g_bDisconnectCB = true;
        sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
        cietEHBS_WidgetConnect(g_sDpName,sPlcName);
        }
}

/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables defined in EHBS faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_WidgetConnect(string deviceName, string sPlcName) {
    int iRes;
    string sSystemName, sAlias,sAgentName;

    dyn_string dsDeviceNameSplit;
    //  check if the device name is of type "PT" if not like of no PLC. -> not available.
    if (dpTypeName(deviceName) != "EHBS")
        g_bUnSystemAlarmPlc = false;

    sSystemName = unGenericDpFunctions_getSystemName(deviceName);

    if (g_bUnSystemAlarmPlc) {
        sAgentName = cietGenericFunctions_getAgentName(deviceName);

        iRes = dpConnect("cietEHBS_WidgetAnimationCB",
                                            deviceName + ".ProcessInput.Data.powerCalc",
                                            deviceName + ".ProcessInput.Data.sensorStatus",
                                            deviceName + ".ST_GTWDEV",
                                            deviceName + ".ProcessInput.Data.setPoint",
                                            deviceName + ".ProcessInput.Data.forcedSetPoint",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
                                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
                                            deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid",
                                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                                            deviceName + ".statusInformation.selectedManager");
        g_bCallbackConnected = (iRes >= 0);
    } else {
        cietEHBS_WidgetDisconnection(g_sWidgetType);
    }
}

/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables defined in EHBS faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_WidgetDisconnect(string deviceName, string sPlcName)
{
    int iRes;
    string sSystemName, sAlias,sAgentName;
    dyn_string dsDeviceNameSplit;
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);

    if (g_bUnSystemAlarmPlc) {
        sAgentName = cietGenericFunctions_getAgentName(deviceName);
        iRes = dpDisconnect("cietEHBS_WidgetAnimationCB",
                            deviceName + ".ProcessInput.Data.powerCalc",
                            deviceName + ".ProcessInput.Data.sensorStatus",
                            deviceName + ".ST_GTWDEV",
                            deviceName + ".ProcessInput.Data.setPoint",
                            deviceName + ".ProcessInput.Data.forcedSetPoint",
                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
                            sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
                            deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid",
                            deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                            deviceName + ".statusInformation.selectedManager");

        g_bCallbackConnected = (iRes >= 0);
    }
}

/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
    . Global variables defined in EHBS faceplate
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/

cietEHBS_WidgetAnimationCB(string sDp_powerCalc, float fpowerCalc,
                            string sDp_sensorStatus, unsigned usensorStatus,
                            string sDp_ST_GTWDEV, bool bST_GTWDEV,
                            string sDp_setPoint, float fsetPoint,
                            string sDp_forcedSetPoint, float fforcedSetPoint,
                            string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue,
                            string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
                            string sDpCodeInvalid, bool bCodeInvalid,
                            string sDpLocked, bool bLocked,
                            string sDpSelectedManager, string sSelectedManager)
{

    string formattedValue, sFormat, sUnit;
    string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor, selectColor;
    string display1Text, display1Color, display2Text, display2Color;
    string Body1Backcolor, Body1Forecolor, Body2Backcolor, Body3Backcolor, Body4Backcolor,  Body4Forecolor, Body5Backcolor,  Body5Forecolor, controlStateColor, overTemperatureColor, overTemperatureLetter;
    bool lockVisible, controlStateVisible,warningVisible, alarmTextVisible, eventStateVisible, overTemperatureVisible;

    sFormat = dpGetFormat(sDp_forcedSetPoint);
    sUnit   = dpGetUnit(sDp_forcedSetPoint);
    display1Color = "unDisplayValue_Status";  // Display 1 -> setPoint / Forced setPoint

    if (getBit(usensorStatus, 3)==1)
        formattedValue = unGenericObject_FormatValue(sFormat, fforcedSetPoint);
    else
        formattedValue = unGenericObject_FormatValue(sFormat, fsetPoint);

    if (getBit(usensorStatus, 13)==0) {
        display1Text=formattedValue+ " "+sUnit;
    } else { //Forced Production RawData
        display1Text="F/RD";
    }

    sFormat = dpGetFormat(sDp_powerCalc);
    sUnit   = dpGetUnit(sDp_powerCalc);
    formattedValue = unGenericObject_FormatValue(sFormat, fpowerCalc);
    display2Text = formattedValue + " " + sUnit;
    display2Color = "unDisplayValue_Output";

    if (getBit(usensorStatus, 3)==1 || getBit(usensorStatus, 13)==1) { // Control State Text -> Forced Mode / Normal Mode
        controlStateVisible=true;
        controlStateColor="unWidget_ControlStateForced";
    } else {
        controlStateVisible=false;
    }


    if (getBit(usensorStatus, 4)==0 && getBit(usensorStatus, 6)==0) { // Card Protection and Heater Over Temperature
        overTemperatureVisible=false;
        overTemperatureColor="cietWidget_CardProtection";
        overTemperatureLetter=" ";
    } else if (getBit(usensorStatus, 4)==1 && getBit(usensorStatus, 6)==0) {
        overTemperatureVisible=true;
        overTemperatureColor="cietWidget_CardProtection";
        overTemperatureLetter=ciet_EHBS_WIDGET_ANIMATION_CARD_OVER_TEMPERATURE_LETTER;
    } else if (getBit(usensorStatus, 4)==0 && getBit(usensorStatus, 6)==1) {
        overTemperatureVisible=true;
        overTemperatureColor="cietWidget_CardProtection";
        overTemperatureLetter=ciet_EHBS_WIDGET_ANIMATION_HEATER_OVER_TEMPERATURE_LETTER;
    } else if (getBit(usensorStatus, 4)==1 && getBit(usensorStatus, 6)==1) {
        overTemperatureVisible=true;
        overTemperatureColor="cietWidget_CardProtection";
        overTemperatureLetter=ciet_EHBS_WIDGET_ANIMATION_HEATER_AND_CARD_OVER_TEMPERATURE_LETTER;
    }

    // Staff not used
    eventStateVisible=false; // event State -> Not used
    alarmTextVisible=false;  // Alarm       -> Not used
    alarmColor="unFaceplate_AlarmActive";
    alarmLetter=" ";

    // BODY ANIMATION
    Body1Backcolor="unWidget_ControlStateForced"; /* unWidget_Background */

    // case for forced production RawData, just one case because we do not take in account the power to animate the widget (empty, half full, full()
    if (getBit(usensorStatus, 13)==1 ) {
        //DebugN("Heater animation -> 20w<powerCalc && Forced Mode");
        Body1Backcolor="unWidget_ControlStateForced";
        Body1Forecolor="unWidget_ControlStateForced";
        Body2Backcolor="unWidget_ControlStateForced";
        Body3Backcolor="unWidget_ControlStateForced";
        Body4Forecolor="unWidget_Background";
        Body5Forecolor="unWidget_Background";
    } else if (fpowerCalc>=ciet_EHBS_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==1 ) {
        //DebugN("Heater animation -> 20w<powerCalc && Forced Mode");
        Body1Backcolor="unWidget_ControlStateForced";
        Body1Forecolor="unWidget_ControlStateForced";
        Body2Backcolor="unWidget_ControlStateForced";
        Body3Backcolor="unWidget_ControlStateForced";
        Body4Forecolor="unWidget_Background";
        Body5Forecolor="unWidget_Background";
    } else if (fpowerCalc>=ciet_EHBS_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==0 ) {
        //DebugN("Heater animation -> 20w<powerCalc && Normal Mode");
        Body1Backcolor="unWidget_ControlStateAuto";
        Body1Forecolor="unWidget_ControlStateAuto";
        Body2Backcolor="unWidget_ControlStateAuto";
        Body3Backcolor="unWidget_ControlStateAuto";
        Body4Forecolor="unWidget_Background";
        Body5Forecolor="unWidget_Background";
    } else if (fpowerCalc>ciet_EHBS_WIDGET_ANIMATION_MINVALUE_powerCalc && fpowerCalc<ciet_EHBS_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==1 ) {
        //DebugN("Heater animation -> 5w<powerCalc>20w && Forced Mode");
        Body1Backcolor="unWidget_ControlStateForced";
        Body1Forecolor="unWidget_ControlStateForced";
        Body2Backcolor="unWidget_ControlStateForced";
        Body3Backcolor="unWidget_Background";
        Body4Forecolor="unWidget_Background";
        Body5Forecolor="unWidget_ControlStateForced";
    } else if (fpowerCalc>ciet_EHBS_WIDGET_ANIMATION_MINVALUE_powerCalc && fpowerCalc<ciet_EHBS_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==0 ) {
        //DebugN("Heater animation -> 5w<powerCalc>20w && Normal Mode");
        Body1Backcolor="unWidget_ControlStateAuto";
        Body1Forecolor="unWidget_ControlStateAuto";
        Body2Backcolor="unWidget_ControlStateAuto";
        Body3Backcolor="unWidget_Background";
        Body4Forecolor="unWidget_Background";
        Body5Forecolor="unWidget_ControlStateAuto";
    } else if (fpowerCalc<=ciet_EHBS_WIDGET_ANIMATION_MINVALUE_powerCalc && getBit(usensorStatus, 3)==1) {
        //DebugN("Heater animation -> powerCalc<5w && Forced Mode");
        Body1Backcolor="unWidget_ControlStateForced";
        Body1Forecolor="unWidget_ControlStateForced";
        Body2Backcolor="unWidget_Background";
        Body3Backcolor="unWidget_Background";
        Body4Forecolor="unWidget_ControlStateForced";
        Body5Forecolor="unWidget_ControlStateForced";
    } else if (fpowerCalc<=ciet_EHBS_WIDGET_ANIMATION_MINVALUE_powerCalc && getBit(usensorStatus, 3)==0) {
        //DebugN("Heater animation -> powerCalc<5w && Normal Mode");
        Body1Backcolor="unWidget_ControlStateAuto";
        Body1Forecolor="unWidget_ControlStateAuto";
        Body2Backcolor="unWidget_Background";
        Body3Backcolor="unWidget_Background";
        Body4Forecolor="unWidget_ControlStateAuto";
        Body5Forecolor="unWidget_ControlStateAuto";
    }

    // WARNING status

    if (getBit(usensorStatus, 12)==1) { // Disable
        display1Color = "cietWidget_Disabled";
        display2Color = "cietWidget_Disabled";
        Body1Backcolor="unWidget_Background";
        Body1Forecolor= "cietWidget_Disabled";
        Body2Backcolor="unWidget_Background";
        Body3Backcolor="unWidget_Background";
        Body4Forecolor= "cietWidget_Disabled";
        Body5Forecolor= "cietWidget_Disabled";

        warningLetter = ciet_WIDGET_TEXT_WARNING_DISABLE;
        warningColor = "cietWidget_Warning";
        warningVisible=true;

        if (getBit(usensorStatus, 0)==1) { // I/O Error
            warningLetter = UN_WIDGET_TEXT_WARNING_ERROR + ciet_WIDGET_TEXT_WARNING_DISABLE;
            warningColor = "cietWidget_Warning";
            warningVisible=true;
        }
    } else {
        if (bCodeInvalid) {
            warningLetter = UN_WIDGET_TEXT_INVALID;
            warningColor = "unDataNotValid";
            warningVisible=true;
            display1Color = "unDataNotValid";
            display2Color = "unDataNotValid";
            Body1Backcolor="unDataNotValid";
            Body1Forecolor="unDataNotValid";
            Body2Backcolor="unWidget_Background";
            Body3Backcolor="unWidget_Background";
            Body4Forecolor="unDataNotValid";
            Body5Forecolor="unDataNotValid";
        } else {
            if (getBit(usensorStatus, 0)==1) { // I/O Error
                warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
                warningColor = "cietWidget_Warning";
                warningVisible=true;
                display1Color = "unFaceplate_AlarmActive";
                display2Color = "unFaceplate_AlarmActive";
            } else {
                warningVisible = false;
                display1Color = "unDisplayValue_Output";
                display2Color = "unDisplayValue_Status";
            }

            if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)) { // Old Data
                warningLetter = UN_WIDGET_TEXT_OLD_DATA;
                warningColor = "unDataNotValid";
                warningVisible=true;
                overTemperatureColor = "unDataNotValid";
                overTemperatureVisible=true;
            }

            if (bST_GTWDEV==0) { // Status Gateway Device
                warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;
                warningColor="cietWidget_Warning";
                warningVisible=true;
            }
        }
    }

    // 3. set the widget.
    unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);

    // 8. Animate widget
    if (g_bSystemConnected)
        setMultiValue("Display1", "text", display1Text,
                        "Display1", "foreCol", display1Color,
                        "eventState", "visible", eventStateVisible,
                        "AlarmText", "visible", alarmTextVisible,
                        "Display2", "text", display2Text,
                        "Display2", "foreCol", display2Color,
                        "Border1", "backCol", Body1Backcolor,
                        "Border1", "foreCol", Body1Forecolor,
                        "Body2", "fill", "[solid]",
                        "Body2", "backCol", Body2Backcolor,
                        "Body3", "fill", "[solid]",
                        "Body3", "backCol", Body3Backcolor,
                        "Body4", "backCol", Body4Backcolor,
                        "Body4", "foreCol", Body4Forecolor,
                        "Body5", "backCol", Body5Backcolor,
                        "Body5", "foreCol", Body5Forecolor,
                        "ControlStateText", "visible", controlStateVisible,
                        "ControlStateText", "foreCol", controlStateColor,
                        "WarningText", "visible", warningVisible,
                        "WarningText", "text", warningLetter,
                        "WarningText", "foreCol", warningColor,
                        "OverTemperatureText", "text", overTemperatureLetter,
                        "OverTemperatureText", "foreCol", overTemperatureColor,
                        "OverTemperatureText", "visible", overTemperatureVisible,
                        "SelectArea", "foreCol", selectColor,
                        "LockBmp", "visible", lockVisible,
                        "WidgetArea", "visible", true);
}

/**
Purpose: Animate widget disconnection

Parameters:
    - sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_WidgetDisconnection(string sWidgetType)
{

//DebugN("WidgetDisconnection EHBS");

    setMultiValue("Display1", "foreCol", "unDataNoAccess",
                    "eventState", "visible", false,
                    "AlarmText", "visible", false,
                    "Display2", "foreCol", "unDataNoAccess",
                    "Border1", "backCol", "unWidget_Background",
                    "Border1", "foreCol", "unDataNoAccess",
                    "Body2", "fill", "[solid]",
                    "Body2", "backCol", "unWidget_Background",
                    "Body3", "fill", "[solid]",
                    "Body3", "backCol", "unWidget_Background",
                    "Body4", "backCol", "unWidget_Background",
                    "Body4", "foreCol", "unDataNoAccess",
                    "Body5", "backCol","unWidget_Background",
                    "Body5", "foreCol", "unDataNoAccess",
                    "ControlStateText", "visible", false,
                    "WarningText", "visible",false,
                    "WarningText", "text", "",
                    "OverTemperatureText", "visible", false,
                    "LockBmp", "visible", false,
                    "WidgetArea", "visible", false );

}

/**
Purpose: Animate widget disconnection

Parameters:
    - sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
    . PVSS version: 3.0
    . operating system: NT and W2000, but tested only under W2000.
    . distributed system: yes.
*/
cietEHBS_PanelWidgetDisconnection(string sWidgetType)
{
    setMultiValue("Body1", "text", "",
                    "WarningText", "text", UN_WIDGET_TEXT_INVALID,
                    "LockBmp", "visible", false);
}

/**
Purpose: configuration of popup menu

Parameters:
    - sDpName, string, input, device name
    - sDpType, string, input, unicos object
    - dsAccessOk, dyn_string, input, authorized action
    - menuList, dyn_string, output, menu to pop-up

Usage: Public

PVSS manager usage: NG, NV

Constraints:
    . Constants UN_POPUPMENU_****
    . PVSS version: 3.0
    . distributed system: yes.
*/
cietEHBS_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

    dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT,
                     ciet_POPUPMENU_RESET,ciet_POPUPMENU_CONFIG,ciet_POPUPMENU_DEADBANDS,
                     ciet_POPUPMENU_FORCED_MODE,ciet_POPUPMENU_AUTO_MODE, ciet_POPUPMENU_SETPOINT,
                     ciet_POPUPMENU_CONSTANTS, ciet_POPUPMENU_CALIBRATION,ciet_POPUPMENU_DISABLE, ciet_POPUPMENU_TENDPARAM, ciet_POPUPMENU_DEBUG,
                     UN_POPUPMENU_FACEPLATE_TEXT, ciet_POPUPMENU_HELP, ciet_POPUPMENU_WIDGET_SIMULATOR, UN_POPUPMENU_TREND_TEXT,
                     ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE);
    int granted;
    bool bForcedMode;
    unsigned uSensorStatus;
    string deviceName;

    int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

    string localManager, sSelectedManager;
    bool bSelected, bLocked;

    //  Selection state
    dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
    dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
    localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
    bSelected = (localManager == "S");

    // Select
    unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    dynAppend(menuList, "SEPARATOR");

    // 1. Reset
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_RESET) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_RESET) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList,"PUSH_BUTTON, Reset, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_RESET)+", " + granted);
    }


    // 2. Config
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONFIG) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONFIG) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, WFip Config,"+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONFIG)+", " + granted);
    }

    // 3. Disable
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_DISABLE) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DISABLE) > 0 ) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Enable/Disable, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DISABLE)+", " + granted);
    }

    // 4. Debug
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0 ) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
    }

    // Separator
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }

    // 5. DeadBands
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEADBANDS) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEADBANDS) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, DeadBands, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEADBANDS)+", " + granted);
    }

    // 6. Set Constants
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONSTANTS) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONSTANTS) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Set Constants, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONSTANTS)+", " + granted);
    }

    // 7. Get Calibration
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_CALIBRATION) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CALIBRATION) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Get Calibration, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CALIBRATION)+", " + granted);
    }

    //11. Tendency
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_TENDPARAM) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_TENDPARAM) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Tendency, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_TENDPARAM)+", " + granted);
    }

  // Separator
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }

    // 8. ForcedMode
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_FORCED_MODE) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_FORCED_MODE) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Forced Mode, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_FORCED_MODE)+", " + granted);
    }

    // 9. Auto Mode
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_AUTO_MODE) > 0) {
        granted = 0;
        if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_AUTO_MODE) > 0) && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, Auto Mode, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_AUTO_MODE)+", " + granted);
    }

    // 10. SetPoint

    deviceName = unGenericDpFunctions_getDpName(sDpName);
    dpGet(deviceName + ".ProcessInput.Data.sensorStatus",uSensorStatus);
    bForcedMode = getBit(uSensorStatus, 3);

    if (dynContains(dsMenuConfig, ciet_POPUPMENU_SETPOINT) > 0) {
        granted = 0;
        if (dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_SETPOINT) > 0 && bForcedMode && bSelected) {
            granted = 1;
        }
        dynAppend(menuList, "PUSH_BUTTON, SetPoint, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_SETPOINT)+", " + granted);
    }

    // Separator
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }
    // Open Agent
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0) {
            granted = 0;
            if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0) {
                    granted = 1;
                }
            dynAppend(menuList,"PUSH_BUTTON, WFIPAgent, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT)+", " + granted);
        }

// Open Segment
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0) {
        granted = 0;
        if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0) {
            granted = 1;
        }
        dynAppend(menuList,"PUSH_BUTTON, WFIPSegment, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT)+", " + granted);
    }

    // Open Crate
    if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_CRATE) > 0) {
        granted = 0;
        if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_CRATE) > 0) {
            granted = 1;
        }
        dynAppend(menuList,"PUSH_BUTTON, Crate, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_CRATE)+", " + granted);
    }

    // Default Unicos action
    unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }

    // 13. Help
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }

    if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0) {
        granted = 0;
        if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0) {
            granted = 1;
        }
        dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
    }

    if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0) {
        granted = 0;
        if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0) {
            granted = 1;
        }
        dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
    }

    // Trend Action
    unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
    if (dynlen(menuList) > 0) {
        if (menuList[dynlen(menuList)] != "SEPARATOR") {
            dynAppend(menuList, "SEPARATOR");
        }
    }
}

/**
Purpose: handle the answer of the popup menu

Parameters:
    - deviceName, string, input, the device dp name
    - sDpType, string, input, the device dp type
    - menuList, dyn_string, input, list of requested action
    - menuAnswer, int, input, selected action from the menuList

Usage: Public

PVSS manager usage: NG, NV

Constraints:
    . Constants UN_POPUPMENU_****
    . PVSS version: 3.0
    . distributed system: yes.
*/
cietEHBS_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
    dyn_string exceptionInfo;
    string descripName, sDpAgentName,sDpGTWName/*, deviceName*/;
    string sAgentName,sBusName;
    dyn_string dsFIPInfos;
    bool bRes;
    int iAgentNumber, iRes;
    float fCurrentSetPoint;

    int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

    unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
    unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
    switch (menuAnswer-iUnicosPopupMenuMin) {
        case ciet_POPUPMENU_ORDER_RESET:                    // Reset
            cietGenericObject_resetDevice(deviceName, exceptionInfo);
            break;
        case ciet_POPUPMENU_ORDER_CONFIG:                   // set config
            ChildPanelOnModal("vision/cietDevices/General/cietSetConfig.pnl",
                          "WFip Conf: " + unGenericDpFunctions_getAlias(deviceName),
                          makeDynString("$sDpName:"+deviceName),0, 0);
            break;

        case ciet_POPUPMENU_ORDER_DISABLE:                  // Disable
            ChildPanelOnModal("vision/cietDevices/General/cietDisable.pnl",
                                  "Disable " + unGenericDpFunctions_getAlias(deviceName),
                                  makeDynString("$sDpName:"+deviceName),0,0);
            break;
        case ciet_POPUPMENU_ORDER_DEBUG:                    // Debug
               cietGenericObject_OpenDebug(deviceName);
            break;
        case ciet_POPUPMENU_ORDER_DEADBANDS:                // DeadBands
            ChildPanelOnModal("vision/cietDevices/General/cietSetDeadBandEH.pnl",
                          "DeadBands: " + unGenericDpFunctions_getAlias(deviceName),
                          makeDynString("$sDpName:"+deviceName), 0, 0);
           break;
        case ciet_POPUPMENU_ORDER_CONSTANTS:                    // Set Constants
            ChildPanelOnModal("vision/cietDevices/General/cietSetConstantsEH.pnl",
                          "Set Constants : " + unGenericDpFunctions_getAlias(deviceName),
                          makeDynString("$sDpName:"+deviceName), 0, 0);
            break;
        case ciet_POPUPMENU_ORDER_CALIBRATION:                  // Get Calibration
             ChildPanelOnModal("vision/cietDevices/General/cietGetCalibration.pnl",
                                  "Calibration Curve:" +  + unGenericDpFunctions_getAlias(deviceName),
                                  makeDynString("$sDpName:"+deviceName),0,0);
            break;
        case ciet_POPUPMENU_ORDER_FORCED_MODE:                  // Forced Mode
            if (dpSetWait(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_FORCEDMODE)<0)
                bRes = false;
            dpGet(deviceName + ".ProcessInput.Data.setPoint",fCurrentSetPoint);
            if (dpSetWait(deviceName + ".ProcessOutput.Command.param1",fCurrentSetPoint)<0)
                bRes = false;
            dpSet(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_FORCEDMODE);

            break;
        case ciet_POPUPMENU_ORDER_AUTO_MODE:                    // Auto Mode
            if (dpSetWait(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_NORMALMODE)<0) bRes = false;
            break;
        case ciet_POPUPMENU_ORDER_SETPOINT:                 // SetPoint
            ChildPanelOnModal("vision/cietDevices/General/cietSetPointEH.pnl",
                              "SetPoint " + unGenericDpFunctions_getAlias(deviceName),
                              makeDynString("$sDpName:"+deviceName),0,0);
            break;
        case ciet_POPUPMENU_ORDER_TENDPARAM:                    // TendParam
            ChildPanelOnModal("vision/cietDevices/General/cietTendParam.pnl",
                              "Tendency " + unGenericDpFunctions_getAlias(deviceName),
                              makeDynString("$sDpName:"+deviceName),0,0);
            break;
        case ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT:                           // Open Agent
            sAgentName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFunctions_getAgentName(deviceName);
            unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sAgentName), exceptionInfo);
            break;
        case ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT:                 // Open Segment
            sBusName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getSegmentName(deviceName);
            unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sBusName), exceptionInfo);
            break;
        case ciet_POPUPMENU_ORDER_OPEN_CRATE:                                       // Open Crate
            cietGenericObject_openCratePanel (deviceName);
            break;
        case ciet_POPUPMENU_ORDER_HELP:                 // Help
            cietGenericObject_OpenHtmlFile("Help.html");
            break;
        case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:                 // Widget Simulator
            ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl", "Widget Simulator","",0, 0);
            break;
        default:
            break;
    }
    // handle the trend action
    unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

    if (dynlen(exceptionInfo) > 0) {
        unSendMessage_toExpertException("Right Click",exceptionInfo);
    }

}


mapping cietEHBS_getDiagnosticByte1Description() {
    // key - bit position; value array {len, title, value0, value1[, value2, value3]}
    mapping diagByteProps;
    diagByteProps[12] = makeDynMixed(1, "Error Stored*", "Heatsink overtemperature", "Normal, no overtemp error", "Error, heatsink overtemp error has occurred");
    diagByteProps[11] = makeDynMixed(1, "Real time", "Thermistor reply selection", "byte B1 does not contain thermistor[3:0] data", "byte B1 includes thremistor[3:0] data");
    diagByteProps[10] = makeDynMixed(1, "Real time", "External watchdog signal has stopped", "Normal, external watchdog signal is present", "Error, external watchsog signal is absent");
    diagByteProps[9] = makeDynMixed(1, "Error Stored*", "ZeroEightREF out of bounds", "Normal, REF signal within range", "Error, REF out of range, relays open");
    diagByteProps[8] = makeDynMixed(2, "Error Stored*", "ErrorCode for ChA Relays", "Normal, no error", "Error on disconnect position check", "Error on AC position check", "Error on DC position check");
    diagByteProps[6] = makeDynMixed(2, "Error Stored*", "ErrorCode for ChB Relays", "Normal, no error", "Error on disconnect position check", "Error on AC position check", "Error on DC position check");
    diagByteProps[4] = makeDynMixed(1, "Error Stored*", "Overtemperature of ChA", "Normal, no overtemperature on ChA", "Error, overtemperature of ChA has occurred");
    diagByteProps[3] = makeDynMixed(1, "Error Stored*", "Overtemperature of ChB", "Normal, no overtemperature on ChB", "Error, overtemperature of ChB has occurred");
    diagByteProps[2] = makeDynMixed(1, "Real time", "ThC/PT selection for ChA", "PCB configuration: ThC sensor of ChA", "PCB configuration: PT sensor of ChA");
    diagByteProps[1] = makeDynMixed(1, "Real time", "ThC/PT selection for ChB", "PCB configuration: ThC sensor of ChB", "PCB configuration: PT sensor of ChB");

    return diagByteProps;
}

mapping cietEHBS_getDiagnosticByte2Description(int diagnosticMode) {
    // key - bit position; value array {len, title, value0, value1[, value2, value3]}
    mapping diagByteProps;

    if (diagnosticMode == 0) {
        diagByteProps[12] = makeDynMixed(2, "Latest error stored*", "ErrorCode for overtemperature of ChA", "Normal, no overtemperature", "Error, temp higher than threshold HARD High", "Error, temp higher than threshold SOFT", "Error, temp lower than threshold HARD Low");
        diagByteProps[10] = makeDynMixed(2, "Latest error stored*", "ErrorCode for overtemperature of ChB", "Normal, no overtemperature", "Error, temp higher than threshold HARD High", "Error, temp higher than threshold SOFT", "Error, temp lower than threshold HARD Low");
        diagByteProps[8] = makeDynMixed(1, "Real time", "Zero cross timeout", "Normal, ZeroCross signal present and within limits", "Error, ZeroCross signal not present or out of limits");
        diagByteProps[7] = makeDynMixed(1, "Real time", "Addressing from comm card is unstable", "Normal, Schb signal is stable", "Error, Schb signal is unstable");
        diagByteProps[6] = makeDynMixed(2, "Real time", "Zero cross jumper selection for ChA", "PCB configuration: Random", "PCB configuration: ZeroCross", "PCB configuration: Peak", "PCB configuration: Not allowed value");
        diagByteProps[4] = makeDynMixed(2, "Real time", "Zero cross jumper selection for ChB", "PCB configuration: Random", "PCB configuration: ZeroCross", "PCB configuration: Peak", "PCB configuration: Not allowed value");
        diagByteProps[2] = makeDynMixed(1, "Error Stored*", "Pending error for ChA that needs to be manually reenabled", "Normal, no error for ChA", "An error has occurred for ChA and it needs to be manually reenabled");
        diagByteProps[1] = makeDynMixed(1, "Error Stored*", "Pending error for ChB that needs to be manually reenabled", "Normal, no error for ChB", "An error has occurred for ChB and it needs to be manually reenabled");
    } else if (diagnosticMode == 1) {
        diagByteProps[12] = makeDynMixed(1, "Real time", "Bit7 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[11] = makeDynMixed(1, "Real time", "Bit6 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[10] = makeDynMixed(1, "Real time", "Bit5 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[9]  = makeDynMixed(1, "Real time", "Bit4 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[8]  = makeDynMixed(1, "Real time", "Bit3 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[7]  = makeDynMixed(1, "Real time", "Bit2 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[6]  = makeDynMixed(1, "Real time", "Bit1 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[5]  = makeDynMixed(1, "Real time", "Bit0 of AC period", "Binary representation of AC period unsigned", "Binary representation of AC period unsigned");
        diagByteProps[4]  = makeDynMixed(1, "Real time", "Setpoint not stable", "Normal, all setpoints are stable", "Error, setpoints are not stable");
        diagByteProps[3]  = makeDynMixed(1, "Real time", "Relay tests are bypassed", "PCB configuration: The relay connectivity tests are enabled", "The relay connectivity tests are bypassed");
        diagByteProps[2]  = makeDynMixed(1, "Error Stored*", "ADC timeout", "Normal, no ADC timeout error", "Error, ADC timeout error has occurred");
        diagByteProps[1]  = makeDynMixed(1, "Real time", "Low priority errors are masked", "PCB configuration for low priority errors [a) Setpoint not stable, b) Schb addressing not stable]: Not masked", "PCB configuration for low priority errors [a) Setpoint not stable, b) Schb addressing not stable]: Masked");
    } else if (diagnosticMode == 2) {
        diagByteProps[12]  = makeDynMixed(1, "Real time", "Bit 5 of hard threshold HIGH for ChA", "PCB configuration: Binary number for threshold HIGH ChA", "PCB configuration: Binary number for threshold HIGH ChA");
        diagByteProps[11]  = makeDynMixed(1, "Real time", "Bit 4 of hard threshold HIGH for ChA", "PCB configuration: Binary number for threshold HIGH ChA", "PCB configuration: Binary number for threshold HIGH ChA");
        diagByteProps[10]  = makeDynMixed(1, "Real time", "Bit 3 of hard threshold HIGH for ChA", "PCB configuration: Binary number for threshold HIGH ChA", "PCB configuration: Binary number for threshold HIGH ChA");
        diagByteProps[9]   = makeDynMixed(1, "Real time", "Bit 2 of hard threshold HIGH for ChA", "PCB configuration: Binary number for threshold HIGH ChA", "PCB configuration: Binary number for threshold HIGH ChA");
        diagByteProps[8]   = makeDynMixed(1, "Real time", "Bit 1 of hard threshold HIGH for ChA", "PCB configuration: Binary number for threshold HIGH ChA", "PCB configuration: Binary number for threshold HIGH ChA");
        diagByteProps[7]   = makeDynMixed(1, "Real time", "isFastRateEnabled", "PCB configuration for data cycling speed: 130msec", "PCB configuration for data cycling speed: 80msec");
        diagByteProps[6]   = makeDynMixed(1, "Real time", "Bit 5 of hard threshold LOW for ChA", "PCB configuration: Binary number for threshold LOW ChA", "PCB configuration: Binary number for threshold LOW ChA");
        diagByteProps[5]   = makeDynMixed(1, "Real time", "Bit 4 of hard threshold LOW for ChA", "PCB configuration: Binary number for threshold LOW ChA", "PCB configuration: Binary number for threshold LOW ChA");
        diagByteProps[4]   = makeDynMixed(1, "Real time", "Bit 3 of hard threshold LOW for ChA", "PCB configuration: Binary number for threshold LOW ChA", "PCB configuration: Binary number for threshold LOW ChA");
        diagByteProps[3]   = makeDynMixed(1, "Real time", "Bit 2 of hard threshold LOW for ChA", "PCB configuration: Binary number for threshold LOW ChA", "PCB configuration: Binary number for threshold LOW ChA");
        diagByteProps[2]   = makeDynMixed(1, "Real time", "Bit 1 of hard threshold LOW for ChA", "PCB configuration: Binary number for threshold LOW ChA", "PCB configuration: Binary number for threshold LOW ChA");
        diagByteProps[1]   = makeDynMixed(1, "Real time", "Bypass manual reenable for ChA", "PCB configuration: After an error, manual reenable is needed", "PCB configuration: after error, operation resumes when error source disappears");
    } else if (diagnosticMode == 3) {
        diagByteProps[12]  = makeDynMixed(1, "Real time", "Bit 5 of hard threshold HIGH for ChB", "PCB configuration: Binary number for threshold HIGH ChB", "PCB configuration: Binary number for threshold HIGH ChB");
        diagByteProps[11]  = makeDynMixed(1, "Real time", "Bit 4 of hard threshold HIGH for ChB", "PCB configuration: Binary number for threshold HIGH ChB", "PCB configuration: Binary number for threshold HIGH ChB");
        diagByteProps[10]  = makeDynMixed(1, "Real time", "Bit 3 of hard threshold HIGH for ChB", "PCB configuration: Binary number for threshold HIGH ChB", "PCB configuration: Binary number for threshold HIGH ChB");
        diagByteProps[9]   = makeDynMixed(1, "Real time", "Bit 2 of hard threshold HIGH for ChB", "PCB configuration: Binary number for threshold HIGH ChB", "PCB configuration: Binary number for threshold HIGH ChB");
        diagByteProps[8]   = makeDynMixed(1, "Real time", "Bit 1 of hard threshold HIGH for ChB", "PCB configuration: Binary number for threshold HIGH ChB", "PCB configuration: Binary number for threshold HIGH ChB");
        diagByteProps[7]   = makeDynMixed(1, "Real time", "'0'", "0", "0");
        diagByteProps[6]   = makeDynMixed(1, "Real time", "Bit 5 of hard threshold LOW for ChB", "PCB configuration: Binary number for threshold LOW ChB", "PCB configuration: Binary number for threshold LOW ChB");
        diagByteProps[5]   = makeDynMixed(1, "Real time", "Bit 4 of hard threshold LOW for ChB", "PCB configuration: Binary number for threshold LOW ChB", "PCB configuration: Binary number for threshold LOW ChB");
        diagByteProps[4]   = makeDynMixed(1, "Real time", "Bit 3 of hard threshold LOW for ChB", "PCB configuration: Binary number for threshold LOW ChB", "PCB configuration: Binary number for threshold LOW ChB");
        diagByteProps[3]   = makeDynMixed(1, "Real time", "Bit 2 of hard threshold LOW for ChB", "PCB configuration: Binary number for threshold LOW ChB", "PCB configuration: Binary number for threshold LOW ChB");
        diagByteProps[2]   = makeDynMixed(1, "Real time", "Bit 1 of hard threshold LOW for ChB", "PCB configuration: Binary number for threshold LOW ChB", "PCB configuration: Binary number for threshold LOW ChB");
        diagByteProps[1]   = makeDynMixed(1, "Real time", "Bypass manual reenable for ChB", "PCB configuration: After an error, manual reenable is needed", "PCB configuration: after error, operation resumes when error source disappears");
    }

    return diagByteProps;
}

